package state51::Job::BulkUpdateRoleAuthority;
use Moose;
use MooseX::Types::Moose qw/ Str Int /;
use Moose::Util::TypeConstraints;
use MooseX::ClassAttribute;
use Text::CSV;
use state51::Role;
use state51::Organisation;

extends 'state51::Job';

class_has documentation => (
    is      => 'ro',
    isa     => 'Str',
    default => 'This is bulk update role.authorities'
);

sub exchange_name { 'bulk_role_authorities' }

has input_file_id => (
    isa           => 'Int',
    is            => 'ro',
    required      => 1,
    documentation => 'Input file in CSV format',
);

has mode => (
    isa      => 'Str',
    is       => 'ro',
    required => 1,
);

sub run {
    my ($self) = @_;

    my $options = {
        binary           => 1,
        sep_char         => ",",
        allow_whitespace => 1,
        quote_char       => '"',
        blank_is_undef   => 1,
    };

    my $csv = Text::CSV->new($options)
      or die "Cannot use CSV: " . Text::CSV->error_diag();


    my $file = state51::Base->storage->load( $self->input_file_id );
    my $content_fh = $file->read;

    state51::Base->connect;
    my $mode = $self->mode;

    my $line = 0;
    while ( my $row = $csv->getline($content_fh) ) {
        $line++;

        next if $line == 1;

        my $name        = $row->[0];
        my $email       = $row->[1];
        my $id          = $mode eq 'update' ? $row->[2] : undef;
        my @org_rep_ids = split( ',', $mode eq 'update' ? $row->[3] : $row->[2] );
        my @auths       = split( ',', $mode eq 'update' ? $row->[4] : $row->[3] );

        next unless $name && $email;

        $self->log_message("$mode role: $email (line: $line)");

        next unless @org_rep_ids && @auths;

        my $role;

        eval {
            if ($id) {
                $role = state51::Role->load($id);
                die "id ($id) not_found" unless $role;
            }

            my @rep_orgs = $mode eq 'update' ? @{ $role->organisation_reps } : ();
            my $changed = 0;
            foreach my $org_id (@org_rep_ids) {
                my $org = state51::Organisation->load_oids($org_id);
                next if grep { $org->id eq $_->id } @rep_orgs;
                push @rep_orgs, $org if $org;
                $changed = 1;
            }

            if ( $mode eq 'update' ) {
                $role->organisation_reps( \@rep_orgs ) if $changed;
            }
            else {
                $role = state51::Role->new(
                    email             => $email,
                    name              => $name,
                    organisation_reps => \@rep_orgs,
                );
                $role->insert();
            }

            foreach my $auth_type (@auths) {
                $auth_type =~ s/^\s+|\s+$//;
                next if $role->has_authority($auth_type);
                $role->add_authority($auth_type);
                $changed = 1;
            }

            $role->upsert if $changed;
        };

        $self->log_message("failed to $mode role: $email (line: $line): $@") if $@;
    }
}

with qw/
  state51::Mixin::JobLogger
  /;

__PACKAGE__->meta->make_immutable;

__PACKAGE__->new_with_options->run unless caller;

1;
