package state51::Job::DownloadAndParseVideoReport;
use Moose;
extends 'state51::Job';

use URI;
use JSON;
use Encode;
use FindBin;
use Storable;
use Text::CSV;
use Date::Calc;
use HTTP::Request;
use LWP::UserAgent;
use URI::Escape qw/uri_escape/;
use Net::Google::DataAPI::Auth::OAuth2;

use state51::Utils qw/ utf8_truncate /;

use state51::Base;

use state51::Logger::Event::JobStarted;
use state51::Logger::Event::JobCompleted;
use state51::Logger::Event::JobFailed;
use state51::Logger::Event::Generic;

BEGIN {
    # NOTE remove this before go-live
    $ENV{DBHOST}          = 'mysql';
    $ENV{DBHOST_READONLY} = 'mysql';
    $ENV{DBUSER}          = 'test';
    $ENV{DBPASS}          = 'test';
    $ENV{DATABASE}        = 'test';
}

has _oauth2 => (
    is  => 'ro',
    isa => 'Net::Google::DataAPI::Auth::OAuth2',
    lazy => 1,
    builder => 'get_google_auth',
);

has _access_token => (
    is  => 'ro',
    isa => 'Net::Google::AccessToken',
    lazy => 1,
    default => sub { Carp::confess;$_->_oauth2->access_token },
);

sub run {
    my ($self) = @_;
    state51::Base->connect;
    my $logger = state51::Logger->new();

    my $file_name = $self->download_file();
    #my $file_name = "$FindBin::Bin/test_video_metadata.csv";
    $logger->message(
        level   => 'info',
        message => [ state51::Logger::Event::Generic->new(
                          message => "job $self downloaded file $file_name",
                     ) ],
    );

    if ($file_name) {
        $self->parse_video_report($file_name);
    }
    $logger->message(
        level   => 'info',
        message => [ state51::Logger::Event::Generic->new(
                         message => "job $self parsed file $file_name",
                     ) ],
    );

    $self->upload_report_to_jssr($file_name);
}

sub parse_video_report {
    my $self = shift;
    my $file_name = shift;

    open my $reportfile_in, "<:utf8", $file_name or die "$file_name: $!";

    my $reportfile_csv_in = Text::CSV->new( { binary => 1, auto_diag => 2} ) or die "Cannot use CSV: ".Text::CSV->error_diag();

    # the grid is the grid of the parent format?
    while ( my $row = $reportfile_csv_in->getline($reportfile_in) ) {
        my (
            $video_id,
            $channel_id,
            $channel_display_name,
            $time_uploaded,
            $time_published,
            $video_title,
            $video_length,
            $views,
            $comments,
            $video_privacy_status,
            $video_url,
            $category,
            $embedding_allowed,
            $ratings_allowed,
            $comments_allowed,
            $claim_origin,
            $content_type,
            $upload_source,
            $claimed_by_this_owner,
            $claimed_by_another_owner,
            $other_owners_claiming,
            $offweb_syndicatable,
            $claim_id,
            $asset_id,
            $custom_id,
            $effective_policy,
            $third_party_video_id,
            $in_video_ads_enabled,
            $third_party_ads_enabled,
            $display_ads_enabled,
            $sponsored_cards_enabled,
            $overlay_ads_enabled,
            $nonskippable_video_ads_enabled,
            $long_nonskippable_video_ads_enabled,
            $skippable_video_ads_enabled,
            $prerolls_enabled,
            $midrolls_enabled,
            $postrolls_enabled,
            $isrc,
            $eidr,
            $is_made_for_kids_user_selection,
            $is_made_for_kids_effective_setting,
            $is_made_for_kids_modifiable
        ) = @{$row};

        next if $isrc eq 'isrc';

        #say join(',',$isrc,$video_id, $channel_id, $asset_id, $video_url);

        my @a_new_identifiers = ();
        my $o_identifier_youtube_asset_id;
        if ($asset_id) {
            $o_identifier_youtube_asset_id =
              state51::Identifier::YouTubeAssetID->get_one($asset_id);
            push @a_new_identifiers, $o_identifier_youtube_asset_id;
        }

        my $o_identifier_youtube_video_id;
        if ($video_id) {
            $o_identifier_youtube_video_id =
              state51::Identifier::YouTubeVideoID->get_one($video_id);
            push @a_new_identifiers, $o_identifier_youtube_video_id;
        }

        my $o_identifier_youtube_channel_id;
        if ($channel_id) {
            $o_identifier_youtube_channel_id =
              state51::Identifier::YouTubeChannelID->get_one($channel_id);
            push @a_new_identifiers, $o_identifier_youtube_channel_id;
        }

        my $o_state51_asset_music_video;
        if ($isrc) {
                ($o_state51_asset_music_video) =
                  state51::Asset::MusicVideo->fetch( isrc => $isrc );
        }
        if ( !$o_state51_asset_music_video ) {
            my $asset_music_video_remote = state51::Asset::MusicVideo->remote;
            my $identifier_youtube_video_remote =
              state51::Identifier::YouTubeVideoID->remote;
            my $filter = ( $asset_music_video_remote->{identifiers}
                  ->includes($identifier_youtube_video_remote) );
            $filter &=
              ( $identifier_youtube_video_remote->{value} eq $video_id );
            my @a_asset_music_videos =
              state51::Asset::MusicVideo->select( $asset_music_video_remote,
                filter => $filter, );
            if ( scalar(@a_asset_music_videos) == 1 ) {
                $o_state51_asset_music_video = $a_asset_music_videos[0];
            }
        }

        if ($o_state51_asset_music_video) {
            my $ra_identifiers = $o_state51_asset_music_video->identifiers;

            my $create_new_youtube_asset_id = 0;
            $create_new_youtube_asset_id = 1 if ($asset_id);

            my $create_new_youtube_video_id = 0;
            $create_new_youtube_video_id = 1 if ($video_id);

            my $create_new_youtube_channel_id = 0;
            $create_new_youtube_channel_id = 1 if ($channel_id);

            my @a_fix_new_identifiers = ();

            foreach my $o_identifier ( @{$ra_identifiers} ) {
                if ( !grep { $_->id eq $o_identifier->id }
                    @a_fix_new_identifiers )
                {
                    push @a_fix_new_identifiers, $o_identifier;
                }
                if ( $create_new_youtube_asset_id
                    && ref($o_identifier) eq
                    'state51::Identifier::YouTubeAssetID' )
                {
                    if ( $o_identifier->value eq $asset_id ) {
                        $create_new_youtube_asset_id = 0;
                    }
                }
                if ( $create_new_youtube_video_id
                    && ref($o_identifier) eq
                    'state51::Identifier::YouTubeVideoID' )
                {
                    if ( $o_identifier->value eq $video_id ) {
                        $create_new_youtube_video_id = 0;
                    }
                }
                if ( $create_new_youtube_channel_id
                    && ref($o_identifier) eq
                    'state51::Identifier::YouTubeChannelID' )
                {
                    if ( $o_identifier->value eq $channel_id ) {
                        $create_new_youtube_channel_id = 0;
                    }
                }
            }
            if ( scalar(@a_fix_new_identifiers) < scalar(@$ra_identifiers) ) {
                $ra_identifiers = \@a_fix_new_identifiers;
                $o_state51_asset_music_video->identifiers(
                    \@a_fix_new_identifiers );
                $o_state51_asset_music_video->update;
            }
            if (   $create_new_youtube_asset_id
                || $create_new_youtube_video_id
                || $create_new_youtube_channel_id )
            {
                my @a_update_identifiers = ();
                if ( scalar @{$ra_identifiers} ) {
                    push @a_update_identifiers, @{$ra_identifiers};
                }
                if ($create_new_youtube_asset_id) {
                    push @a_update_identifiers, $o_identifier_youtube_asset_id;
                }
                if ($create_new_youtube_video_id) {
                    push @a_update_identifiers, $o_identifier_youtube_video_id;
                }
                if ($create_new_youtube_channel_id) {
                    push @a_update_identifiers,
                      $o_identifier_youtube_channel_id;
                }
                $o_state51_asset_music_video->identifiers(
                    \@a_update_identifiers );
                $o_state51_asset_music_video->update;
            }
            if ($isrc) {
                my $o_state51_asset_recording;
                eval {
                    $o_state51_asset_recording =
                      state51::Asset::Recording->fetch_one( isrc => $isrc );
                };
                if ($o_state51_asset_recording) {
                    my $ra_supports = $o_state51_asset_music_video->supports;
                    my @a_checked_supports = grep { $_ } @{$ra_supports};
                    if ( scalar(@a_checked_supports) != scalar(@$ra_supports) )
                    {
                        $o_state51_asset_music_video->supports(
                            \@a_checked_supports );
                        $o_state51_asset_music_video->update;
                        $ra_supports = $o_state51_asset_music_video->supports;
                    }
                    if ( scalar(@$ra_supports) ) {
                        if ( !grep { $_->id eq $o_state51_asset_recording->id }
                            @{$ra_supports} )
                        {
                            my @a_new_supports = ();
                            push @a_new_supports, @{$ra_supports};
                            push @a_new_supports, $o_state51_asset_recording;
                            $o_state51_asset_music_video->supports(
                                \@a_new_supports );
                            $o_state51_asset_music_video->update;
                        }
                    }
                }
            }
        }
        else {
            my $o_state51_asset_recording;
            if ($isrc) {
                eval {
                    $o_state51_asset_recording =
                      state51::Asset::Recording->fetch_one( isrc => $isrc );
                };
            }
            my $ra_artists = [];
            if ($o_state51_asset_recording) {
                $ra_artists = $o_state51_asset_recording->artists;
            }

            my $truncated_video_title = utf8_truncate($video_title, 255);

            $video_length ||= 1;
            $o_state51_asset_music_video = state51::Asset::MusicVideo->new(
                title       => $truncated_video_title,
                length      => $video_length,
                identifiers => \@a_new_identifiers,
                ( $isrc                ? ( isrc    => $isrc )       : () ),
                ( scalar(@$ra_artists) ? ( artists => $ra_artists ) : () ),
                (
                    $o_state51_asset_recording
                    ? ( supports => [$o_state51_asset_recording] )
                    : ()
                ),
            );
            $o_state51_asset_music_video->insert;
        }

        # https://www.youtube.com/watch?v=zu-tpuE-QCg
        # ?v={video id}

        # find or insert a state51::Asset with the right identifers
    }
    $reportfile_csv_in->eof or $reportfile_csv_in->error_diag();
    close $reportfile_in;
}

sub download_file {
    my $self = shift;
    my $access_token = $self->_access_token;

    my $ua     = LWP::UserAgent->new;
    my $parser = JSON->new;

    my $job_id = $self->get_job_id();
    my $job_url;
    if ($job_id) {
        $job_url =
            'https://youtubereporting.googleapis.com/v1/jobs/'
          . $job_id
          . '/reports';
        my $job_uri = URI->new($job_url);
        my $body    = { "onBehalfOfContentOwner" => '-wxiPpVkxs5fxMasHMpJXw', };
        my %q       = (%$body);
        $job_uri->query_form( \%q );
        my $request = HTTP::Request->new( GET => $job_uri );
        $request->header( 'Authorization', sprintf "%s %s",
            $access_token->token_type, $access_token->access_token );
        my $response     = $ua->request($request);
        my $rh_response  = $parser->decode( $response->content );
        my $download_url = $rh_response->{reports}[0]{downloadUrl};
        print "\ndownload_url:" . $download_url . "\n";

        if ($download_url) {
            my $download_uri = URI->new($download_url);
            $request = HTTP::Request->new( GET => $download_uri );
            $request->header( 'Authorization', sprintf "%s %s",
                $access_token->token_type, $access_token->access_token );
            $response = $ua->request($request);
            if ( $response->is_success ) {

                #print "\nresponse:" . Dumper($response) . "\n";
                print "--> downloaded $download_url, saving it to file";

                my @a_today = Date::Calc::Today;

                my $year        = sprintf( '%4d', $a_today[0] );
                my $month       = sprintf( '%2d', $a_today[1] );
                my $day         = sprintf( '%2d', $a_today[2] );
                my $date_string = "$year$month$day";
                my $file_name   = '/tmp/content_owner_video_metadata_a2_'
                  . $date_string . '.csv';

                # save the file content to disk
                open my $fh, '>:utf8', $file_name or die "ERROR: $!n";
                print $fh $response->decoded_content;
                close $fh;

                print "nsaved file:n";
                print "-------------n";
                print "filename:  " . $response->filename . "\n";
                print "size:      " . ( -s $file_name ) . "\n";
                return $file_name;
            }
            else {
                my $error_msg = "ERROR: download of $download_url failed: "
                  . $response->status_line;
                die "$error_msg\n";
            }
        }
    }
}

sub upload_report_to_jssr {
    my $self = shift;
    my $file_name = shift;

    my $ua     = LWP::UserAgent->new;
    my $parser = JSON->new;

    my $job_url =
      'http://uni-bear.int.openimp.com/api/expectations/1189/windows';
    my $job_uri = URI->new($job_url);

    my $request = HTTP::Request->new( GET => $job_uri );

    my $response = $ua->request($request);
    if ( $response->is_success ) {
        my $rh_response    = $parser->decode( $response->content );
        my $ra_windows     = $rh_response->{data}{windows};
        my @a_sort_windows = sort { $b->{id} <=> $a->{id} } @$ra_windows;
        my $id             = $a_sort_windows[0]->{id};
        my $file_path      = $file_name;
        my $job_url =
          'http://uni-bear.int.openimp.com/api/windows/' . $id . '/files';
        my $response = $ua->post(
            $job_url,
            Content_Type => 'form-data',
            Content      => [
                file => [$file_path],
            ]
        );
        if ( $response->is_success ) {
            print "\nupload of $file_path succeed\n";
            unlink($file_name) or die "Can't delete $file_name: $!\n";
        }
        else {
            my $error_msg = "ERROR: upload of $file_path failed: "
                . $response->status_line;
            die $error_msg;
        }
    }
}

sub get_job_id {
    my $self = shift;
    my $access_token = $self->_access_token;

    my $ua     = LWP::UserAgent->new;
    my $parser = JSON->new;

    my $url = 'https://youtubereporting.googleapis.com/v1/jobs';
    my $uri = URI->new($url);

    my $body = {
        "includeSystemManaged"   => 'True',
        "onBehalfOfContentOwner" => '-wxiPpVkxs5fxMasHMpJXw',
    };

    my %q = (%$body);
    $uri->query_form( \%q );

    my $request = HTTP::Request->new( GET => $uri );
    $request->header( 'Authorization', sprintf "%s %s",
        $access_token->token_type, $access_token->access_token );

    my $response    = $ua->request($request);
    my $rh_response = $parser->decode( $response->content );

    my $job_id;
    foreach my $rh_job ( @{ $rh_response->{jobs} } ) {
        if ( $rh_job->{reportTypeId} eq 'content_owner_video_metadata_a2' ) {
            $job_id = $rh_job->{id};
            last;
        }
    }
    return $job_id;
}

sub get_google_auth {
    my $self = shift;

    # Authentication code based on example from gist at
    #  https://gist.github.com/hexaddikt/6738247

    # Get the token that we saved previously in order to authenticate:
    my $session_filename =
"$FindBin::Bin/var/stored_youtube_google_access.session";

    # Be sure to edit the lines below to fill in your correct client
    # id and client secret!
    my $oauth2 = Net::Google::DataAPI::Auth::OAuth2->new(
        client_id =>
'526380923466-6tockgum459lfcjsipk5o91uer1hjabb.apps.googleusercontent.com',
        client_secret => 'GJ8FUcBqUl_58jGvLP42VLV_',
        scope         => [
            'https://www.googleapis.com/auth/yt-analytics-monetary.readonly',
            'https://www.googleapis.com/auth/yt-analytics.readonly'
        ],
    );

    ## Deserialize the file so we can thaw the session and reuse the refresh token
    my $session = retrieve($session_filename);
    #
    my $restored_token = Net::OAuth2::AccessToken->session_thaw(
        $session,
        auto_refresh => 1,
        profile      => $oauth2->oauth2_webserver,
    );

    $oauth2->access_token($restored_token);
    return $oauth2;
}

with qw/
    state51::Mixin::JobLogger
/;

__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
