package state51::Job::EmailTemplateToRole;
use Moose;
use MooseX::Types::Moose qw/ Str Int /;
use Moose::Util::TypeConstraints;
use MooseX::ClassAttribute;
use MooseX::Types::Email qw/EmailAddress/;
use state51::Logger;

extends 'state51::Job';

class_has documentation => (
    is  => 'ro',
    isa => 'Str',
    default =>
'A general job to email a template to a given role.',
);

sub exchange_name {
    'email_template_to_role';
}

has 'role_id' => (
    is            => 'ro',
    isa           => 'Int',
    required      => 1,
    documentation => 'The id of the role to be emailed.',
);

has 'source_id' => (
    is       => 'ro',
    isa           => 'Int',
    required => 0,
    documentation =>
      'The optional id of the source to be used in the context of the mail.',
);

has 'url' => (
    is       => 'ro',
    isa      => 'Str',
    required => 0,
    documentation =>
'The callback URL to be used in the email.',
);

has 'email_template_id' => (
    is       => 'ro',
    isa      => 'Int',
    required => 1,
    documentation =>
      'The full instance id of the email template to be used in response to the user.',
);

has 'subject' => (
    is       => 'ro',
    isa      => 'Str',
    required => 1,
    documentation =>
'The subject to be used in the email.',
);

sub suppress_console_display { 1 }

sub run {
    my ($self) = @_;
    state51::Base->connect;

    my $role = state51::Role->load( $self->role_id );

    die "invalid email address: ".$role->email unless is_EmailAddress($role->email);

    my $source;
    if ($self->source_id) {
        $source = state51::Source->load( $self->source_id );
    }

    $self->send_email(
        $role,
        $self->email_template_id,
        $self->url,
        $source,
        $self->subject,
    );

  # FIXME: new event needed?
#    state51::Logger->new->message(
#    level   => 'trace',
#    message => [
#        state51::Logger::Event::SignupStateChange->new(
#            current_value  => 'new',
#            organisation_id => $organisation->id,
#            role_id         => $role->id,
#        )
#    ]
#);

    return 1;
}

sub send_email {
    my ($self, $role, $template_id, $url, $source, $subject) = @_;

    # original signup template id: 85786940840354
    my $mail_from = 'no-reply@state51.com';

    my $source_name;
    if ($source) {
        $source_name = $source->name;
    }

    my $mail = state51::Mail::Send->new(template_id => $template_id);
    $mail->mail(
        {   to       => $role->email,
            from     => $mail_from,
            subject  => $subject,
            content_type => 'text/html; charset="UTF-8"',
            content_transfer_encoding => 'quoted-printable',
            bounce    => $role->secret . 'a' . $role->uniq. '@bounces.openimp.com',
            p        => {
                role      => $role,
                url       => $url,
                mail_from => $mail_from,
                source_name => $source_name,
            },
        }
    );
}


with qw/
  state51::Mixin::JobLogger
  /;

__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
