package state51::Job::EraseReleaseFormatAndGraph;
use Moose;
use Try::Tiny qw/ try catch /;
use state51::Logger;
use Try::Tiny qw/ try /;
use MooseX::Types::Moose qw/ Int Str /;
use feature ':5.10';

extends 'state51::Job';

sub exchange_name { 'erasereleaseformatandgraph' }

use namespace::autoclean;

has releaseformat_id => (
    is       => 'ro',
    isa      => Int,
    required => 1,
);

has cooldown_type => (
    is => 'ro',
    isa => Str,
    default => 'erase_releaseformat_and_graph',
);

my %seen;

# Arrays aren't updated when the thing they reference is erased :-(

sub run {
    my ($self) = @_;

    state51::Base->connect;

    # can't just return as we've ACK-ed the message, go slow though
    if ($self->in_cooldown) {
        print STDERR "erase_releaseformat_and_graph cooldown is enabled, sleeping.\n";
        sleep(60);
    }

    my $release_format = state51::ReleaseFormat->load($self->releaseformat_id);

    my $related = $self->find_instances_relating_to($release_format);

    #say "found:";
    #foreach my $instance (@{$related}) {
    #    say $instance->id . " " . $instance->as_string;
    #}

    # related is a giant list of instances relating to and including the release format
    # each of these methods will deal with those instances it knows how to manage
    # things can be removed from related and not erased, they will survive untouched
    # things can be erased and removed from related
    # the order of the methods is significant
    # everything must be removed from related by the time we reach the end.

    # FIXME: what about Use/Relationships relating to tracks?

    $related = $self->delete_export_items($related);
    $related = $self->delete_file_importfile($related);
    $related = $self->safe_delete_packshots_and_thumbnails($related);
    $related = $self->delete_files_encoded_by_us($related);
    $related = $self->delete_filetransforms($related);
    $related = $self->delete_remaining_files($related);
    $related = $self->delete_use($related);
    $related = $self->delete_relationships($related);
    $related = $self->delete_deliver_desires($related);
    $related = $self->delete_contributions($related);
    $related = $self->delete_qa_flags($related);
    $related = $self->delete_identifiers($related);
    $related = $self->safe_delete_batches($related);
    $related = $self->safe_delete_import_requests($related);
    $related = $self->safe_delete_export_requests($related);
    $related = $self->delete_supp_formats($related);
    $related = $self->delete_tracks($related);
    $related = $self->delete_recordings($related);
    $related = $self->safe_delete_publishers($related);

    $self->delete_release_format($release_format);

    $related = $self->delete_release($related);
    $related = $self->safe_delete_artists($related);
    $related = $self->safe_delete_label($related);

    # explode if there's anything left
    say scalar(@{$related});
    my %class;
    foreach my $instance (@{$related}) {
        $class{$instance->class_table}++;
    }
    use DDP;
    p %class;
}

sub logged_erase {
    my ($self, $instance) = @_;

    return unless $instance->is_persistent;

    $self->logger->message(
        level   => "info",
        message => [
            state51::Logger::Event::Generic->new(
                message => "erasing instance id : " . $instance->id
            )
        ]
    );
    $instance->erase();
}

sub safe_delete_packshots_and_thumbnails {
    my ($self, $related) = @_;

    my @result;
    foreach my $instance (@{$related}) {
        if ($instance->isa('state51::File::Image')) {
            my @formats_using_this = state51::ReleaseFormat->fetch_readonly( packshot => $instance );
            # exclude dervived formats
            @formats_using_this = grep {$_->barcode} @formats_using_this;
            if ($instance->encoded_by_us() or (scalar(@formats_using_this) == 1)) {
                $self->logged_erase($instance);
            }
        }
        else {
            push @result, $instance;
        }
    }
    return \@result;
}

sub delete_files_encoded_by_us {
    my ($self, $related) = @_;

    my @result;
    foreach my $instance (@{$related}) {
        if (    $instance->isa('state51::File')
            and $instance->can('encoded_by_us')
            and $instance->encoded_by_us)
        {
            $instance->request(undef);
            $instance->update();
            $self->logged_erase($instance);
        }
        else {
            push @result, $instance;
        }
    }
    return \@result;
}

sub delete_filetransforms {
    my ($self, $related) = @_;

    my @result;
    foreach my $instance (@{$related}) {
        if ($instance->isa('state51::FileTransform')) {
            $self->logged_erase($instance);
        }
        else {
            push @result, $instance;
        }
    }
    return \@result;
}

# this will be master audio and master packshot
sub delete_remaining_files {
    my ($self, $related) = @_;

    my @result;
    foreach my $instance (@{$related}) {
        if ($instance->isa('state51::File')) {
            $self->logged_erase($instance);
        }
        else {
            push @result, $instance;
        }
    }
    return \@result;
}

sub delete_use {
    my ($self, $related) = @_;

    my @result;
    foreach my $instance (@{$related}) {
        if ($instance->isa('state51::Use')) {
            $self->logged_erase($instance);
        }
        else {
            push @result, $instance;
        }
    }
    return \@result;
}

sub delete_relationships {
    my ($self, $related) = @_;

    my @result;
    foreach my $instance (@{$related}) {
        if ($instance->isa('state51::Relationship')) {
            $self->logged_erase($instance);
        }
        else {
            push @result, $instance;
        }
    }
    return \@result;
}

sub delete_file_importfile {
    my ($self, $related) = @_;

    my @result;
    foreach my $instance (@{$related}) {
        if ($instance->isa('state51::File::ImportFile')) {

            # update the array to remove me
            my ($import_batch) = state51::ImportBatch->fetch(
                import_file => [$instance],
                includes    => 'import_file',
            );
            if ($import_batch) {
                $import_batch->import_file(
                    [
                        grep { defined($_) and $_->id != $instance->id }
                          @{$import_batch->import_file()}
                    ]
                );
                $import_batch->update();
            }
            $instance->request(undef);
            $instance->update();

            $self->logged_erase($instance);
        }
        else {
            push @result, $instance;
        }
    }
    return \@result;
}

sub delete_export_items {
    my ($self, $related) = @_;

    my @result;
    foreach my $instance (@{$related}) {
        if ($instance->isa('state51::ExportItem')) {
            my ($export_batch) = state51::ExportBatch->fetch(
                export_item => [$instance],
                includes    => 'export_item'
            );
            if (($export_batch) and (scalar(@{$export_batch->export_item()}))) {
                $export_batch->export_item(
                    [
                        grep { defined $_ and $_->id != $instance->id }
                          @{$export_batch->export_item()}
                    ]
                );
            }
            $self->logged_erase($instance);
        }
        else {
            push @result, $instance;
        }
    }
    return \@result;
}

sub delete_deliver_desires {
    my ($self, $related) = @_;

    my @result;
    foreach my $instance (@{$related}) {
        if ($instance->isa('state51::DeliverDesire')) {
            $self->logged_erase($instance);
        }
        else {
            push @result, $instance;
        }
    }
    return \@result;
}

sub delete_qa_flags {
    my ($self, $related) = @_;

    my @result;
    foreach my $instance (@{$related}) {
        if ($instance->isa('state51::QAFlag')) {
            $self->logged_erase($instance);
        }
        else {
            push @result, $instance;
        }
    }
    return \@result;
}

sub delete_contributions {
    my ($self, $related) = @_;

    my @result;
    foreach my $instance (@{$related}) {
        if ($instance->isa('state51::Contribution')) {
            $self->logged_erase($instance);
        }
        else {
            push @result, $instance;
        }
    }
    return \@result;
}

sub delete_identifiers {
    my ($self, $related) = @_;

    my @result;
    foreach my $instance (@{$related}) {
        if ($instance->isa('state51::Identifier')) {
            $self->logged_erase($instance);
        }
        else {
            push @result, $instance;
        }
    }
    return \@result;
}

# you must update these arrays when you delete the items elsewhere
# tangram doesn't do it for you, and you'll never know they are empty
sub safe_delete_batches {
    my ($self, $related) = @_;

    my @result;
    foreach my $instance (@{$related}) {
        if ($instance->isa('state51::ImportBatch')) {
            if (scalar(@{$instance->import_file()} == 0)) {
                $self->logged_erase($instance);
            }
        }
        elsif ($instance->isa('state51::ExportBatch')) {
            if (scalar(@{$instance->export_item()} == 0)) {
                $self->logged_erase($instance);
            }
        }
        else {
            push @result, $instance;
        }
    }
    return \@result;
}

sub safe_delete_import_requests {
    my ($self, $related) = @_;

    my @result;
    foreach my $instance (@{$related}) {
        if ($instance->isa('state51::Request::Import')) {
            my @related_batches =
              state51::ImportBatch->fetch(to_fulfil => $instance);
            if (scalar(@related_batches) == 0) {
                $self->logged_erase($instance);
            }
        }
        else {
            push @result, $instance;
        }
    }
    return \@result;
}

sub safe_delete_export_requests {
    my ($self, $related) = @_;

    my @result;
    foreach my $instance (@{$related}) {
        if ($instance->isa('state51::Request::Export')) {
            my @related_batches =
              state51::ExportBatch->fetch(to_fulfil => $instance);
            if (scalar(@related_batches) == 0) {
                $self->logged_erase($instance);
            }
        }
        else {
            push @result, $instance;
        }
    }
    return \@result;
}

sub safe_delete_publishers {
    my ($self, $related) = @_;

    my @result;
    foreach my $instance (@{$related}) {
        if ($instance->isa('state51::Asset::Publisher')) {
            next unless $instance->is_persistent;

            # TODO: Asset does this anyway when you erase, but be explicit
            # should is_orphan be safe_to_erase
            # and also exist on import/export batch, artists etc?
            if ($instance->is_orphan()) {
                $self->logged_erase($instance);
            }
        }
        else {
            push @result, $instance;
        }
    }
    return \@result;
}

sub safe_delete_artists {
    my ($self, $related) = @_;

    my @result;
    foreach my $instance (@{$related}) {
        if ($instance->isa('state51::Artist')) {
            my @assets = state51::Asset->fetch(
                artists  => $instance,
                includes => 'artists'
            );
            my @formats =
              state51::Format->fetch(artist => $instance, includes => 'artist');

            my $line_remote = state51::Line->remote();
            my $filter      = ($line_remote->{artists}->includes($instance));
            my $lines = state51::Base->storage->count($line_remote, $filter);

            my @org =
              state51::Organisation->fetch(represents_artist => $instance);

            if (
                (scalar(@assets) + scalar(@formats) + $lines + scalar(@org)) ==
                0)
            {
                $self->logged_erase($instance);
            }
        }
        else {
            push @result, $instance;
        }
    }
    return \@result;
}

sub safe_delete_label {
    my ($self, $related) = @_;

    my @result;
    foreach my $instance (@{$related}) {
        if ($instance->isa('state51::OrganisationAttribute::Label')) {
            my @assets = state51::Asset->fetch(label => $instance,);
            my @formats = state51::ReleaseFormat->fetch(label => $instance);
            my @lines = state51::Line->fetch(label => $instance);
            if ((scalar(@assets) + scalar(@formats) + scalar(@lines)) == 0) {
                $self->logged_erase($instance);
            }
        }
        else {
            push @result, $instance;
        }
    }
    return \@result;
}

sub delete_supp_formats {
    my ($self, $related) = @_;

    my @result;
    foreach my $instance (@{$related}) {
        if ($instance->isa('state51::Format::Supplementary')) {
            $self->logged_erase($instance);
        }
        else {
            push @result, $instance;
        }
    }
    return \@result;
}

sub delete_recordings {
    my ($self, $related) = @_;

    my @result;
    foreach my $instance (@{$related}) {
        if ($instance->isa('state51::Asset')) {

            # do nothing, recordings that were safe to delete
            # have been erased when the track was erased.
        }
        else {
            push @result, $instance;
        }
    }
    return \@result;
}

sub delete_tracks {
    my ($self, $related) = @_;

    my @result;
    foreach my $instance (@{$related}) {
        if ($instance->isa('state51::Trackv2')) {
            $self->logged_erase($instance);
        }
        else {
            push @result, $instance;
        }
    }
    return \@result;
}

sub delete_release {
    my ($self, $related, $release_format) = @_;

    my @result;
    foreach my $instance (@{$related}) {
        if ($instance->isa('state51::Release')) {

            # do nothing, erased by rf already
        }
        else {
            push @result, $instance;
        }
    }
    return \@result;
}

# RF actually has a nice "before" modifier on it's erase
# which duplicates a lot of what we're doing here
# but it's better to log it here rather than use that magic
sub delete_release_format {
    my ($self, $release_format) = @_;

    my $image_id;
    if (($release_format->release) and ($release_format->release->packshot)) {
        eval {
            $image_id = $release_format->release->packshot->id
              if $release_format->release->packshot;
        };
        if ((defined $release_format->release->packshot) and (not $image_id)) {

            # packshot is a broken reference, and will break the format->erase()
            $release_format->release->packshot(undef);
            $release_format->release->update;
        }
    }
    eval {
        $image_id = $release_format->packshot->id
          if $release_format->packshot;
    };
    if (    ($release_format->multivolume_format)
        and ((defined $release_format->packshot) and (not $image_id)))
    {

        # packshot is a broken reference, and will break the format->erase()
        $release_format->packshot(undef);
        $release_format->update;
    }

    $self->logged_erase($release_format);
}

# really "find all instances caused by my existence"
sub find_instances_relating_to {
    my ($self, $release_format) = @_;

    my @result;

    $self->logger->message(
        level   => "info",
        message => [
            state51::Logger::Event::Generic->new(
                message => "Finding everything related to "
                  . $release_format->as_string
            )
        ]
    );

    my @tracks = @{$release_format->tracks()};
    push @result, @tracks;

    foreach my $track (@tracks) {
        next unless defined $track;
        next unless defined $track->recording;

        push @result, $track->recording
          unless exists $seen{$track->recording->id};
        $seen{$track->recording->id}++;

        my @audio = state51::File::Audio->fetch(
            track            => $track,
            asset_controller => $release_format->asset_controller
        );
        push @result, @audio;

        if (@audio) {
            push @result, state51::FileTransform->fetch(master => \@audio);
            push @result, state51::FileTransform->fetch(result => \@audio);
        }

        push @result,
          state51::Contribution->fetch(asset => [ $track, $track->recording ]);

        push @result, @{$track->identifiers};

        foreach my $artist (@{$track->recording->artists}) {
            push @result, $artist
              unless exists $seen{$artist->id};
            $seen{$artist->id}++;
        }

        foreach my $publisher (@{$track->recording->publishers}) {
            push @result, $publisher unless $seen{$publisher->id};
            $seen{$publisher->id}++;
        }
    }
    push @result, state51::Contribution->fetch(asset => $release_format);
    if ($release_format->can('identifiers')) {
        push @result, @{$release_format->identifiers};
    }
    foreach my $artist (@{$release_format->artists}) {
        next unless defined $artist;
        push @result, $artist
          unless exists $seen{$artist->id};
        $seen{$artist->id}++;
    }

    my @import_file = state51::File::ImportFile->fetch(
        release_format => [$release_format],
        includes       => 'release_format'
    );
    push @result, @import_file;

    if (@import_file) {
        my @import_batch = state51::ImportBatch->fetch(
            import_file => \@import_file,
            includes    => 'import_file',
        );
        push @result, @import_batch;

        foreach my $batch (@import_batch) {
            push @result, $batch->to_fulfil
              unless exists $seen{$batch->to_fulfil->id};
            $seen{$batch->to_fulfil->id}++;
        }
    }

    my @export_item = state51::ExportItem->fetch(
        release_format => [$release_format],
        includes       => 'release_format'
    );
    push @result, @export_item;
    if (@export_item) {
        my @export_batch = state51::ExportBatch->fetch(
            export_item => \@export_item,
            includes    => 'export_item'
        );
        push @result, @export_batch;

        foreach my $batch (@export_batch) {
            push @result, $batch->to_fulfil
              unless exists $seen{$batch->to_fulfil->id};
            $seen{$batch->to_fulfil->id}++;
        }
    }

    # non-multi volume formats where part > 1 route packshot to the first format
    # which sometimes doesn't exist
    if (
        (
            $release_format->multivolume_format == 1
            or (not defined($release_format->multivolume_format)
                and $release_format->volume_part == 1)
        )
        and $release_format->packshot
      )
    {
        push @result, $release_format->packshot;
        my @image_transforms =
          state51::FileTransform->fetch(master => $release_format->packshot);
        push @result, @image_transforms;
        push @result, map { $_->result() } @image_transforms;
    }

    if ($release_format->label()) {
        push @result, $release_format->label
          unless exists $seen{$release_format->label->id};
        $seen{$release_format->label->id}++;
    }
    if ($release_format->release) {
        push @result, $release_format->release
          unless exists $seen{$release_format->release->id};
        $seen{$release_format->release->id}++;
    }

    push @result, state51::DeliverDesire->fetch(asset => $release_format);

    push @result,
      state51::ExportItem->fetch(
        release_format => [$release_format],
        includes       => 'release_format'
      );

    push @result, state51::QAFlag->fetch(release_format => $release_format);

    # use actually knows how to get rid of the relationships
    push @result, state51::Use->fetch(asset => $release_format);

    if ($release_format->isa('state51::ReleaseFormat')) {
        foreach my $supp_format (
            state51::Format::Supplementary->fetch(parent => $release_format))
        {
            push @result, $supp_format;
            push @result, @{$self->find_instances_relating_to($supp_format)};
        }
    }

    push @result,
      state51::Relationship::Limited->fetch(
        asset => $release_format,
        type  => 'RetailLicensor'
      );

    @result = grep { defined $_ } @result;

    return \@result;
}

sub lockname {
    my ($self) = @_;

    return "erase_releaseformat_" . $self->releaseformat_id
}

with qw/
  state51::Script::Helper::InteractiveSuccess
  state51::Mixin::HasLogger
  state51::Mixin::RunsAsyncJobs
  state51::Mixin::JobSignature
  state51::Mixin::LockedJob
  state51::Mixin::CanBeCooled
  /;

__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;

