package state51::Job::FixBrokenFile;
use Moose;
use File::Temp;
use LWP::Simple;
use Compress::Zlib ();
use MooseX::Types::Moose qw/ ArrayRef Num Str /;
use Try::Tiny;
use Module::Find qw/ useall /;
use List::Util qw/max/;

BEGIN {
    useall state51::Job::FixBrokenFile::Action;
}

extends 'state51::Job';

has 'fix' => (
    isa           => 'Bool',
    is            => 'ro',
    default       => 0,
    documentation => 'Attempt to fix broken files',
);

has 'oid' => (
    isa           => ArrayRef,
    is            => 'ro',
    documentation => 'OIDs to check'
);

has tempdir => (
    is      => 'ro',
    isa     => Str,
    default => sub {
        return File::Temp::tempdir( DIR => ( $ENV{FILE_CACHE_DIR} || "/tmp" ) );
    },
    lazy   => 1,
    traits => [qw/ DoNotSerialize /],
);

sub exchange_name { 'fixbrokenfile' }

sub run {
    my ($self) = @_;
    state51::Base->connect;

    foreach my $oid ( @{ $self->oid } ) {
        my $file;
        my $ft;

        $self->log_message( "Attempted to fix $oid", "info" );

        if ( $oid =~ /0131$/ ) {
            $ft   = state51::FileTransform->load($oid);
            $file = $ft->master;
        }
        elsif ( $oid =~ /0127$/ ) {
            $file = state51::File::Video->load($oid);
        }
        elsif ( $oid =~ /0033$/ ) {
            $file = state51::File::Audio->load($oid);
        }
        else {
            die "Bad oid: $oid";
        }

        my $action;
        $action ||= $self->check_track($ft) if ($ft);

        # Make sure the copies in Mogile are consistent first, as
        # validate_contents may see the broken one.
        $action ||= $self->check_sha1s($file);

        my $file_is_bad;
        try {
            $file->validate_contents();
        }
        catch {
            $file_is_bad = $_ || "unknown error";
        };

        if ($file_is_bad) {
            $action ||= $self->check_rederive($file);

            # These tests all assume that the SHA1 is correct, otherwise
            # all bets are off.
            $action ||= $self->check_crc32($file);
            $action ||= $self->check_filesize($file);
            $action ||= $self->check_encoding($file);

            if ($ft) {
                $action ||=
                  state51::Job::FixBrokenFile::Action::FileTransform::Leave
                  ->new(
                    filetransform => $ft,
                    message       => "File is bad, but I cannot see how.",
                  );
            }

            # Special case if we have a file and no transform
            $action ||=
              state51::Job::FixBrokenFile::Action::FileTransform::Leave->new(
                file    => $file,
                message => "File is bad, but I cannot see how.",
              );

        }
        else {
            $action ||= $self->check_release_live($ft) if ($ft);

            if ($ft) {
                $action ||=
                  state51::Job::FixBrokenFile::Action::FileTransform::Leave
                  ->new(
                    filetransform => $ft,
                    message =>
                      "Master is apparently good, yet transform still failed.",
                  );
            }
            $action ||= state51::Job::FixBrokenFile::Action::File::Leave->new(
                file => $file,
                message =>
                  "Master is apparently good, yet transform still failed.",
            );
        }

        $action->do_action( $self->fix );

        $action->log_action( $self->logger );

        if ( $action->do_action ) {
            try {
                $action->action();
            }
            catch {
                if ($ft) {
                    $self->log_message( "Fixing " . $ft->id . " exploded: $_",
                        "error" );
                }
                else {
                    $self->log_message( "Fixing " . $file->id . " exploded: $_",
                        "error" );
                }
            };
        }

        # Clean up
        opendir( my $dir, $self->tempdir ) or die $!;
        while ( my $d = readdir($dir) ) {
            if ( $d =~ /^\d+__\d+$/ ) {
                unlink( $self->tempdir . "/$d" ) or warn $!;
            }
        }
        closedir $dir;
    }
    return;
}

sub check_track {
    my ( $self, $ft ) = @_;

    my $track;
    try {
        $track = $ft->target_track();
    }
    catch {
        $self->log_message("Could not get track: $_");
    };

    unless ($track) {
        return state51::Job::FixBrokenFile::Action::FileTransform::Junk->new(
            filetransform => $ft,
            message => "Encoding has no target_track, it cannot complete.",
        );
    }
    return;
}

sub check_sha1s {
    my ( $self, $file ) = @_;

    my $filepref = $self->tempdir . "/" . $file->id . "__";

    unless ( $file->digest ) {
        return state51::Job::FixBrokenFile::Action::File::Junk->new(
            file => $file,
            message =>
"No SHA1 for this file, giving up.  File metadata is clearly eaten.",
        );
    }

    my $copies = 0;
    my @urls   = $file->mogile_paths();
    if ( !@urls ) {
        return state51::Job::FixBrokenFile::Action::File::Leave->new(
            file    => $file,
            message => "Got no mogile paths for file.",
        );
    }
    foreach my $u (@urls) {
        $self->log_message( "Grabbing URL $u", "trace" );
        if ( getstore( $u, $filepref . $copies ) == 200 ) {
            $copies++;
        }
        else {
            warn "URL failed to check OK: $u";
            warn "$u contains localhost so your environment is retarded"
              if $u =~ m/127\.0\.0\.1/;
        }
    }
    if ( $copies == 0 ) {
        return state51::Job::FixBrokenFile::Action::File::Leave->new(
            file    => $file,
            message => "Could not download any copies.",
        );
    }

    my $good_copy;
    my $bad_copy;

    for ( my $i = 0 ; $i < $copies ; $i++ ) {
        open( my $fh, "<", $filepref . $i ) or die $!;
        binmode $fh;
        my $dg = Digest::SHA1->new()->addfile($fh)->b64digest;
        if ( $dg eq $file->digest ) {
            $self->log_message( "Copy $i is good", "trace" );
            $good_copy = $i;
        }
        else {
            $self->log_message( "Copy $i is bad", "trace" );
            $bad_copy = $i;
        }
    }

    if ( defined $good_copy ) {
        if ( defined $bad_copy ) {
            return state51::Job::FixBrokenFile::Action::File::ReUpload->new(
                path    => $filepref . $good_copy,
                message => "One bad copy in Mogile, one good copy.",
                file    => $file,
            );
        }
    }
    else {
        return state51::Job::FixBrokenFile::Action::File::Junk->new(
            file => $file,
            message =>
"All copies in Mogile are bad copies, you need to resupply the audio.",
        );
    }
}

sub check_crc32 {
    my ( $self, $file ) = @_;

    my $calced = $file->calc_crc32();

    if ( $file->crc32 != $calced ) {
        $self->log_message( "Fixing crc32", "trace" );
        return state51::Job::FixBrokenFile::Action::File::MashCRC32->new(
            file    => $file,
            message => "File had incorrect crc32",
            crc32   => $calced,
        );
    }
    return;
}

sub check_filesize {
    my ( $self, $file ) = @_;

    my $calced = $file->calc_filesize();

    if ( $file->filesize != $calced ) {
        $self->log_message( "Fixing filesize", "trace" );
        return state51::Job::FixBrokenFile::Action::File::MashFileize->new(
            file     => $file,
            message  => "File had incorrect filesize",
            filesize => $calced,
        );
    }
    return;
}

sub check_encoding {
    my ( $self, $file ) = @_;

    # This only really makes sense for a master
    my $newfile = state51::File->new_from_file( $file->local_file_path );
    my $enc     = $newfile->encoding();
    if ( $file->encoding->id ne $enc->id ) {
        return state51::Job::FixBrokenFile::Action::File::ChangeEncoding->new(
            file     => $file,
            message  => "Encoding wasn't set correctly.",
            encoding => $enc,
        );
    }
    elsif ( $file->mimetype ne $newfile->mimetype ) {
        return state51::Job::FixBrokenFile::Action::File::ChangeMIME->new(
            file       => $file,
            message    => "MIME type wasn't set correctly.",
            mime_minor => $newfile->mime_minor,
            mime_major => $newfile->mime_major,
        );
    }

    return;
}

sub check_rederive {
    my ( $self, $file ) = @_;

    if ( $file->am_transformed ) {
        return state51::Job::FixBrokenFile::Action::File::Rederive->new(
            message =>
              "Broken file was a derived file, so we are re-requesting it.",
            file => $file,
        );
    }
    return;
}

sub check_release_live {
    my ( $self, $ft ) = @_;

    my $bad;

    try {
        $ft->target_track->releaseformat->assert_meta_valid() or die "???";
    }
    catch {
        $bad = $_ || "unknown fail";
        if ( $bad =~ /Format must have a price band/ ) {
            $self->log_message( "Ignoring priceband error: $bad", "info" );
            $bad = undef;
        }
    };

    $self->log_message( "error so far: $bad", "info" ) if $bad;

    if ($bad) {
        return state51::Job::FixBrokenFile::Action::FileTransform::Leave->new(
            message =>
"The metadata was not valid, this needs to be fixed before this track will encode.  The error reported was;\n $bad",
            filetransform => $ft,
        );
    }
    return;
}

sub DEMOLISH {
    my ($self) = @_;
    system( "rm", "-rf", $self->tempdir ) == 0
      or warn "couldn't remove tempdir";
    return;
}

=pod

=head2 Actions

Check all the copies in Mogile.  Do any have the wrong check sum?  Do any
have the right check sum?  Fix if necessary.

Update the crc32 if necessary.

=cut

__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;

