package state51::Job::FixCMYKPackshots;
use state51::Moose;
use MooseX::Storage;
with Storage (format => "JSON");

use MooseX::Types::Moose qw/ Int ArrayRef /;
use state51::Mail::Send;
use File::Slurp;
use Text::CSV;

extends 'state51::Job';

sub exchange_name { 'fixbrokenfile' }

method lockname() { 'fixbrokenfile'.$self->id }

has id => (
    isa         => Int,
    is          => 'ro',
    required    => 1,
);

has job_control_file_id => (
    isa         => Int,
    is          => 'ro',
    required    => 1,
);


method run() {
    state51::Base->connect;
    my $email_body;
    my $attachment_text = "barcode\told packshot id\tnew packshot id\tstatus\n";

    my ($format_hash_ref, $asset_controller, %format_hash, $asset_controller_identifier);
    eval {
        ($format_hash_ref, $asset_controller) = $self->parse_job_control_file(state51::File::Text->load($self->job_control_file_id));
        $asset_controller_identifier = $asset_controller->identifier;
        %format_hash = %$format_hash_ref;
    };
    if ($@) {
        $attachment_text = "Failed to parse job control file to ReleaseFormat list and asset controller because ".$@;
        $asset_controller_identifier ="UNKNOWN";
        %format_hash = ();
    }

    my $succeeded = 0;
    my $failed = 0;
    my $old_packshot_id = "UNKNOWN";
    foreach my $barcode (keys %format_hash) {
        eval {
            my $format = $format_hash{$barcode};
            die "$barcode not in catalogue for '$asset_controller_identifier'"
                unless $format;
            my $old_packshot = $format->packshot;
            $old_packshot_id = $old_packshot->id;
            die "exisiting packshot for $barcode already converted to RGB colourspace"
                if ($old_packshot->description && $old_packshot->description =~ /Generated by state51::Job::FixCMYKPackshots/);
            my $new_packshot = $self->reencode_packshot($format);
            die "new packshot for $barcode isn't tagged appropriately"
                unless ($new_packshot->description =~ /Generated by state51::Job::FixCMYKPackshots/);
            $format->replace_packshot($new_packshot);
            $format->update;
            $attachment_text .= "$barcode\t$old_packshot_id\t".$new_packshot->id."\tOK\n";
            $succeeded += 1;
        };
        if ($@) {
            my $error_cause = $@;
            $error_cause =~ s/\t/ /g;
            $error_cause =~ s/[\n\r]/ /g;
            $attachment_text .= $barcode."\t$old_packshot_id\tFAILED\t$error_cause\n";
            $failed += 1;
        }
    }
    my $hostname = `hostname`;
    $hostname =~ s/[\n\r]//g;
    $email_body = "Attempted to replace ".($failed+$succeeded)." packshots for $asset_controller_identifier on $hostname.\n\n$succeeded packshots updated successfully.\n$failed packshots failed to update.\n\nPlease see attached CSV for summary";


    my $mail = state51::Mail::Send->new();
    $mail->mail({
        to              => 'rick@openimp.com',
        from            => 'jobworker_'.`hostname`.'@openimp.com',
        subject         => "Replaced packshots for $asset_controller_identifier",
        content_type    => 'text/plain; charset="utf-8"',
        raw_output      => $email_body,
        attachment      => $attachment_text,
        filename        => "replaced_packshots-".$asset_controller_identifier,
        attachment_type => 'text/csv',
    });
}

use File::Temp qw/ tempdir /;
method reencode_packshot() {
    my $format = shift;

    my $old_packshot_file_name = $format->packshot->local_file_path;

    my $tempdir = File::Temp::tempdir( CLEANUP => 1, TMPDIR => 1 );
    my $new_packshot_file_name =  $tempdir."/new_packshot.tiff";
    system("convert $old_packshot_file_name -colorspace sRGB -compress lzw $new_packshot_file_name");
    my $new_packshot_file_object = state51::File::Image->new_from_file($new_packshot_file_name);
    $new_packshot_file_object->description("Generated by state51::Job::FixCMYKPackshots from ".$format->packshot->id);
    $new_packshot_file_object->insert();
    $new_packshot_file_object->write();
    return $new_packshot_file_object;
}



method parse_job_control_file() {
    my $job_control_file_object = state51::File::Text->load($self->job_control_file_id);
    my $job_control_file_name = $job_control_file_object->local_file_path;

    my $csv = Text::CSV->new({ sep_char => ',' });
    my @lines = read_file( $job_control_file_name, binmode => ':utf8');
    my ($org, $org_identifier);
    if ($csv->parse($lines[0]) ) {
        my @fields = $csv->fields;
        $org_identifier = $fields [0];
        $org = state51::Organisation->fetch_one(identifier => $org_identifier);
    }
    shift @lines;
    my %formats;
    foreach my $line (@lines) {
        if ($csv->parse ($line)) {
            my @fields = $csv->fields;
            my $barcode = $fields [0];
            eval {
                my $format = state51::ReleaseFormat->fetch_one( status => 'live', asset_controller => $org, barcode => $barcode);
                $formats{$barcode}= $format;
            };
            if ($@) {
                $formats{$barcode} = undef;
            }
        }
        else {
            die "couldn't parse control file";
        }
    }
    return (\%formats, $org);
}

with qw/
    
    state51::Mixin::Loggable
    state51::Mixin::LockedJob
    state51::Mixin::UnloadAfterJob
/;


__PACKAGE__->meta->make_immutable;
1;

