package state51::Job::FixFileCephStatus;
use Moose;

use state51::File;

extends 'state51::Job';

# This job should be called with arrays of instance ids
# each one will be checked for existence in Ceph and Mogile
# Mogile files will be ignored for now.
# We will attempt to make the file available in Ceph with a backend of ceph_sha512
# This will mean the instance will become "deduped" in ceph
# The existing copy will be immediately erased, not "trashed".

has 'ids' => (
    isa           => 'ArrayRef',
    is            => 'ro',
    documentation => 'IDs to check'
);

sub exchange_name { 'fixfilecephstatus' }

sub run {
    my ($self) = @_;
    state51::Base->connect;

    foreach my $id ( @{ $self->ids } ) {
        $self->log_message( "Attempted to fix $id", "info" );

        my $file = state51::File->load_oids($id)
            or die "Bad id: $id";
        $self->fix_ceph_status($file);
    }
    return;
}

sub file_set_available {
    my ($self, $file) = @_;
    $file->storage_backend('ceph_sha512');
    # FIXME: why is this via the backend?
    $file->backend->stored_in_ceph(1);
    $file->update_stored_in_ceph; # set striped status properly
    $file->update;
}

sub file_upload_and_set_available {
    my ($self, $file) = @_;
    my $sha = $file->sha512;
    $file->backend->ceph->oid($sha);        # replace oid with sha
    $file->backend->ceph->has_duplicate(0); # force actual write
    $file->backend->write_ceph;             # rewrite file to sha
    # sha should now check ok, otherwise something has gone wrong, bail out
    $file->backend->ceph->is_downloadable($sha)
        or die "File check on sha failed after write for: $sha";
    $self->file_set_available($file);
}

sub fix_ceph_status {
    my ($self, $file) = @_;
    my $sha = $file->sha512;
    my $oid = $file->id;

    if ($file->backend->ceph->is_downloadable($sha)) {
        $self->file_set_available($file);
    }
    elsif ($file->backend->ceph->is_downloadable($oid)) {
        # copy exists on ceph as oid = id
        $file->backend->ceph->oid($oid);   # replace sha with oid
        $file->backend->download
            or die "Could not download a local copy from Ceph";
        $self->file_upload_and_set_available($file);
    }
    elsif ($file->backend->mogile->is_downloadable) {
    # XXX do not copy from mogile until we have space
    # NOTE that we don't want to flag Mogile-available files as gone
    #    # copy exist on Mogiie
    #    $file->backend->mogile->download
    #        or die "Could not download a local copy from Mogile";
    #    $self->file_upload_and_set_available($file);
    }
    else {
        # file is not in ceph or mogile
        $file->stored_in_ceph(0);
        # `stored` has a trigger which will re-request transforms if we're a result
        # so don't do this...
        # $file->stored(state51::File::FILE_STATUS_GONE);
        # maybe erase_only_me?
        $file->update;
    }

    # tidy up the oid if we've been deduped
    if ($file->backend->ceph->is_downloadable($sha) and
        $file->backend->ceph->is_downloadable($oid)) {
        $file->backend->ceph->_erase_for_real($oid);
    }
}

=head2 Actions

This job attempts to move everything to the Ceph deduplication backend, and make the C<storage_backend> and C<stored_in_ceph> columns always make sense.

Check that the sha512 exist on Ceph.
 If so, set the C<storage_backend> to 'ceph_sha512' and C<stored_in_ceph> to 1.
 If not, check if either the oid exists on Ceph, or the Mogile file can be downloaded.
  If so, upload that file to the sha512 on Ceph, and update C<storage_backend>/C<stored_in_ceph> as above.
  If not, set C<stored_in_ceph> to 0, and C<stored> to C<FILE_STATUS_GONE>, skip the next step.
Then delete the oid on Ceph if it exists.

=cut

__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;

