package state51::Job::ImportRequestQAOverviewNotify;
use state51::Moose;
use MooseX::Storage;
with Storage (format => "JSON");

use MooseX::Types::Moose qw{Bool Str};
use state51::Base;

extends 'state51::Job';

sub BUILD {} #  / Script::Helper::DB conflict err

has 'no_op' => (
    isa         => Bool,
    is          => 'ro',
    traits      => [qw{Getopt}],
    cmd_flag    => 'no-op',
    cmd_aliases => 'n',
);

has 'requests' => (
    isa     => Str,
    is      => 'ro',
    traits  => [qw{Getopt}],
    cmd_aliases => 'r',
);

method run() {
    my $storage = state51::Base->connect;

    $self->log_message( message => ["Running " . __PACKAGE__ . "\n"] );

    my @import_requests;

    if( my $req_str = $self->requests ) {

        my @req_ids = split(/,/, $req_str);
        foreach my $id (@req_ids) {
            push @import_requests, state51::Request::Import->load($id);
        }
    }
    else {
        # TODO: This should only find requests from CI clients

        my $req_remote = state51::Request::Import->remote;
        my $org_remote = state51::Organisation->remote;
        my $orgattrci_remote =
            state51::OrganisationAttribute::CiPlatformUser->remote;

        my $filter = (
            ( $req_remote->{status} eq 'complete' )
            & ( $req_remote->{qa_notified} == 0 )
            & ( $req_remote->{organisation} eq $org_remote )
            & ( $orgattrci_remote->{organisation} eq $org_remote )
        );

        @import_requests = state51::Base->select(
            $req_remote, $filter
        );
    }

    unless (scalar @import_requests) {
        return;
    }

    $self->log_message(
        ($self->no_op ? '[No-op]: ' : '')
        . "Total of " . scalar @import_requests
        . " Import Requests to review", 'info' );

    my $rfmt_remote = state51::ReleaseFormat->remote;

    @import_requests = sort {$b->create_time cmp $a->create_time} @import_requests;

    REQUEST:
    foreach my $request (@import_requests) {
        # FIXME - this can be removed once metadata update requests set their
        # completion_date
        next if ($request->notes && $request->notes =~ /^Metadata file upload order/);

        my $filter = (($rfmt_remote->{to_fulfil}->includes($request))
            & ($rfmt_remote->{status} ne 'junk')
            & ($rfmt_remote->{status} ne 'withdrawn')
            & ($rfmt_remote->{status} ne 'pending')
            & (
                ($rfmt_remote->{qa_flags_run} eq undef)
                | ($rfmt_remote->{qa_flags_run} < $request->completion_date)
              )
        );

        my @formats = $storage->select($rfmt_remote, $filter);

        # All have had qaflags run since the Import Request completed
        if (not scalar @formats) {
            $self->log_message(
                ($self->no_op ? '[No-op]: ' : '')
                . "Requesting QA notification for Import Request '"
                . $request->id . "'", 'info'
            );

            # Will set qa_notified = 1
            $request->send_qa_overview_email(no_op => $self->no_op);

            next REQUEST;
        }
        else {

            my $fids = join(',', map { $_->id } @formats);

            $self->log_message(
                "Request " . $request->id . " (" . $request->create_time . ")"
                . " has formats still awaiting QA, requesting: $fids", 'info'
            );

            # Don't care if formats are cached for processing in the slow
            # queue; what matters is that we do them in the fast queue soon
            $self->cache_delete($_->qa_cache_flag_key) for @formats;
            $_->request_qa('runqaforformatimport') for @formats;
        }
    }
}

with qw{
    state51::Mixin::UnloadAfterJob
    state51::Mixin::HasLogger
    state51::Mixin::Loggable
    state51::Mixin::JobSignature
};

__PACKAGE__->new_with_options->run unless caller;
1;
