package state51::Job::KickOffDeliveriesForOrder;
use state51::Base;
use Function::Parameters;
use Moose;
use Try::Tiny;
use MooseX::Types::Moose qw/ Int Bool /;
use MooseX::Storage;
with Storage (format => "JSON");
use state51::Job::DeliverOneDPForOrder;
use Moose::Autobox;
use namespace::autoclean;
use state51::SQLHacks qw/ are_all_dds_finished_for_this_dp /;

extends 'state51::Job';

sub exchange_name { 'order_delivery_kickoff' }

has force_automatic_export => (
    is => 'ro',
    isa => Bool,
    default => 0,
    documentation => 'Override the DPs preference for if it is automatically exportable',
);

method run() {
    my $order = $self->_order_object;

    my $has_error;
    return 1 if ($order->status eq "complete");
    if (! $order->deliver_desires_generated ) {
        $has_error = $self->_do_comment_and_return('Deliver desires have not been generated yet, cannot deliver');
    }
    if (! $order->transforms_requested ) {
        $has_error = $self->_do_comment_and_return('Transforms have not been requested yet, cannot deliver');
    }

    if ($order->require_service_metadata_xls_validation and not defined $order->service_metadata_processed) {
        return $self->_do_comment_and_return('The service metadata xls attached has not been processed succesfully, please fix any error found before continue');
    }

    if ( $order->order_failed_to_be_deliverable ) {
        $has_error = $self->_do_comment_and_return('Not all formats are exportable, please correct errors and trigger a new checking process at '.
          $order->ci_support_uri.' before trying delivery ');
    }
    elsif (! $order->all_formats_exportable ) {
        $self->run_job('state51::Job::CheckAnOrderIsReadyForDelivery',
            order => $self->order,
        );
        return $self->_do_comment_and_return('Products on the order have not been checked yet, a checking process has been triggered');
    }

    if ($has_error) {
        $self->update_ticket_status('open');
        return $has_error;
    }

    my $comment = '';
    my $deliveries_started = '';
    my $deliveries_not_started = '';

    my $already_started = ! ! $order->automatic_delivery_job_started;
    $order->automatic_delivery_job_started(state51::Base->dbnow);
    $order->update;

    my @musicservices = $order->musicservice_destinations->flatten;

    my @deliveries_to_make;
    foreach my $dp (@musicservices) {
        my @td = state51::TransferDetails->fetch( organisation => $order->organisation, destination => $dp );
        if (scalar(@td) != 1) {
            $self->log_message( level => "warn", message => ["Order id:" . $self->order . ". Wrong number of transfer details for ".$dp->tla]);
        }
        elsif ($td[0]->status eq 'on-hold') {
            $self->log_message( level => "warn", message => ["Order id:" . $self->order . ". Skipping on hold dp ".$dp->tla]);
            next;
        }

        # "KickOff" is also called on orders which have been running for a while
        # try and avoid re-making DeliverOneDPForOrder when there's nothing to do
        my $work_to_do = state51::SQLHacks::are_all_dds_finished_for_this_dp($order, $dp);
        next unless $work_to_do;

        push(@deliveries_to_make, $dp);
        $deliveries_started .= sprintf("\n - %s (%s) ",$dp->name, $dp->tla);
    }

    $comment = sprintf("%s Kicking off deliveries for order %s", $self->date, $self->order);

    if ($deliveries_started) {
        $comment .= sprintf("\nDelivery Job started automatically for: %s", $deliveries_started);
    }

    if ($deliveries_not_started) {
        $has_error = 1;
        $comment .= sprintf("\nDeliveries for Services below can not be processed automatically yet, please run manual delivery process for them: %s",
            $deliveries_not_started);
    }

    if (!$already_started) {
        $self->_do_comment_and_return($comment);
    }
    $self->update_ticket_status('open') if $has_error;

    $self->_deliver_to_dp($_) for @deliveries_to_make;

    return 1;
}

method _do_comment_and_return ($comment) {
    print $comment if $self->interactive;

    return 1;
}

method _deliver_to_dp($dp) {
    $self->run_job('state51::Job::DeliverOneDPForOrder',
        order => $self->order,
        delivery_point_id => $dp->id,
        async => 1, # Force always async here, or we start delivering all orders
                    # from CLI
    );
}

with qw/
    state51::Script::Helper::InteractiveSuccess
    state51::Mixin::HandleOrdersOnJobs
    state51::Mixin::RunsAsyncJobs
    state51::Mixin::JobSignature
    state51::Mixin::HasLogger
/;

#
__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
