package state51::Job::MigrateToCeph512;
use Moose;
use MooseX::Types::Moose qw/ Str /;
use Moose::Util::TypeConstraints;
use MooseX::ClassAttribute;
use state51::Logger;
extends 'state51::Job';

local $ENV{CEPH_DEDUPLICATION} = 1;

class_has documentation => (
    is  => 'ro',
    isa => 'Str',
    default =>
'Called with a sha512 digest, it finds all files with that digest and the ceph_oid backend and deduplicates them.',
);

sub exchange_name {
    'migrate_to_ceph512';
}

has 'sha' => (
    is       => 'ro',
    isa      => 'Str',
    required => 0,
    documentation =>
'The sha512 of the files with the ceph_oid backend to be deduplicated to ceph_sha512.',
);

has cooldown_type => (
    is => 'ro',
    isa => Str,
    default => 'migrate_to_ceph512',
);

sub suppress_console_display { 1 }

sub deduplicate_sha {
    my ($self, $sha) = @_;

    # FIXME: what about all the files where stored_in_ceph=1 but storage_backend is not ceph_oid?
    my @ceph_oid_files = state51::File->fetch({
        status          => 'live',
        storage_backend => 'ceph_oid',
        sha512          => $sha,
    });
    return unless @ceph_oid_files;

    my $log = sprintf("Removing %i files with sha %s\n", scalar(@ceph_oid_files), $sha);
    $self->log_message(
        message => [$log],
        level => "info",
    );

    my $needs_upload =
        !$ceph_oid_files[0]->backend->ceph->is_downloadable($sha);

    my $i = 1;
    foreach my $file (@ceph_oid_files) {
        # download the first file to local temp storage, just in case we don't have a ceph_sha512 copy
        if ($i == 1 and $needs_upload) {
            $log = sprintf("Moving first file %i to %s\n", $file->id, $sha);
            $self->log_message(
                message => [$log],
                level => "info",
            );
            $file->backend->download
                or die "Could not download a local copy";
            $file->backend->ceph->oid($sha);        # replace oid with sha
            $file->backend->ceph->has_duplicate(0); # force actual write
            $file->backend->write_ceph;             # rewrite file to sha
            # sha should now check ok, otherwise something has gone wrong, bail out
            $file->backend->ceph->is_downloadable($sha)
                or die "File is_downloadable on sha failed after write for: $sha";
        }
        my $id = $file->id;
        $log = sprintf("Deleting file %i: %i\n", $i, $id);
        $self->log_message(
            message => [$log],
            level => "info",
        );
        $file->backend->ceph->_erase_for_real($id)   # call internal ceph delete on old oid
            if $file->backend->ceph->is_downloadable($id);
        if ($i > 1 or !$needs_upload) {
            $file->log_message( message => [state51::Logger::Event::CephDeduplicate->new(
                size => $file->filesize,
                key => $sha,
            )], level => "info");
        }
        $file->storage_backend('ceph_sha512');  # update storage backend
        $file->update;
        $file->unload();

        $log = sprintf("File %i updated to ceph_sha512\n", $id);
        $self->log_message(
            message => [$log],
            level => "info",
        );
        $i++;
    }
}

sub run {
    my ($self) = @_;
    state51::Base->connect;

    return unless $self->sha and length($self->sha) == 86;

    $self->deduplicate_sha($self->sha);
    # go easy on the ceph cluster
    my $sleep_duration = $ENV{CEPH_MIGRATE_SLEEP} || 60;
    sleep($sleep_duration);

    return 1;
}

with qw/
  state51::Mixin::JobLogger
  state51::Mixin::CanBeCooled
  /;

__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
