package state51::Job::RPC;
use Moose;
use MooseX::Types::Moose qw/ Str /;

use JSON qw/decode_json/;
extends 'state51::Job';

has 'method_stream' => (
    isa           => Str,
    is            => 'ro',
    documentation => 'A block of json containing methods to call.',
    required      => 1,
);

sub exchange_name {
    'job_rpc';
}

sub suppress_console_display { 1 }

sub run {
    my ($self) = @_;
    state51::Base->connect;

    # these die, which will turn into a JobFailed event
    my $method_stream = decode_json($self->method_stream);
    $self->validate_method_stream(\$method_stream);
    $self->authenticate(\$method_stream);
    $self->authorise(\$method_stream);
    my $result = $self->run_method_stream(\$method_stream);

    return $result;
}

sub validate_method_stream {
    my ($self, $method_stream) = @_;

    die "invalid message_version"
        unless $$method_stream->{message_version} eq '20220809';
    die "invalid schema"
        unless $$method_stream->{schema} eq '20220809';
    die "missing methods"
        unless $$method_stream->{methods};

    # FIXME: preserve over multiple calls
    # TODO: Get this from a schema instead of directly from the classes?
    my $job_class_list = state51::Base::global_job_class_list();
    my %allowed_class = map { $_ => 1 } @{$job_class_list};

    foreach my $method (@{$$method_stream->{methods}}) {
        die "missing method"
            unless $method->{method};
        die "missing params"
            unless exists $method->{parameters};

        my $fully_qualified_method_name = $method->{method};
        my $params = $method->{parameters};

        # TODO: class methods are not yet supported
        my ($class_name,$method_name) = split(/->/, $fully_qualified_method_name, 2);

        die "class $class_name not allowed"
            unless $allowed_class{$class_name};

        my $job_accessors = $class_name->job_accessors();
        my %allowed_job_accessor = map { $_ => 1 } @{$job_accessors};

        my $job_methods = $class_name->job_methods();

        die "method <<$method_name>> not allowed on class <<$class_name>> : $fully_qualified_method_name"
            unless $allowed_job_accessor{$method_name} || $job_methods->{$method_name};

        # TODO: validate parameters
    }

    return 1;
}

# TODO:
sub authenticate {
    my ($self, $method_stream) = @_;
    return 1;
}

# TODO:
sub authorise {
    my ($self, $method_stream) = @_;
    return 1;
}

sub run_method_stream {
    my ($self, $method_stream) = @_;

    my $context; # current instance to run methods on
    my $context_class; # class of the current instance
    my $previous_class = '';
    # set by load or new
    # You can call methods on multiple different classes / instances in a single stream
    # but you can't interleave them
    # so class A->load, A->method, B->load, B->method is fine

    my $job_accessors;
    my %allowed_job_accessor;
    my %ref_fields;
    my %array_fields;

    foreach my $method (@{$$method_stream->{methods}}) {
        my $fully_qualified_method_name = $method->{method};
        my $params = $method->{parameters};

        my ($class_name, $method_name) = split(/->/, $fully_qualified_method_name, 2);

        if ($previous_class ne $class_name) {
            $job_accessors = $class_name->job_accessors();
            %allowed_job_accessor = map { $_ => 1 } @{$job_accessors};
            %ref_fields = map { $_ => 1 } $class_name->ref_fields();
            %array_fields = map { $_ => 1 } $class_name->array_fields();

            $previous_class = $class_name;
        }

        # deal with simple accessors
        if ($allowed_job_accessor{$method_name}) {
            # if the $class->params field has type ref in the schema,
            # set the params to ->load_oids($params->[0])
            # else params just becomes $params->[0]

            if (exists $ref_fields{$method_name} and ref $params eq 'ARRAY' and @$params == 1) {
                $params = state51::Base->load_oids($params->[0]);
            }
            elsif (exists $array_fields{$method_name} and ref $params eq 'ARRAY') {
                my @loaded_instances;
                foreach my $param (@$params) {
                    push @loaded_instances,state51::Base->load_oids($param);
                }
                $params = \@loaded_instances;
            }
            else {
                $params = $params->[0];
            }
        }
        # methods go via an rpc_job_$method to allow for parameter re-mapping
        else {
            $method_name = 'rpc_job_'.$method_name;
        }

        #warn "calling $class_name->$method_name";

        # allow us to set undef
        if ($params eq 'undef') {
            $params = undef;
        }

        # validate the method name
        my $method = $class_name->can($method_name);
        die "method $method_name not found on class $class_name"
            unless $method;

        if (($method eq 'rpc_job_load') or ($method eq 'rpc_job_new') or ($method eq 'rpc_job_load_oids')) {
            $context = $class_name->$method_name($params);
            $context_class = $class_name;
        }
        elsif ($method eq 'transform_type') {
            $context = $class_name->$method_name($params);
            # assumes that the params to transform_type are the class name
            $context_class = $params;
        }
        elsif ($context) {
            if ($context_class ne $class_name) {
                die "cannot call $method_name on $class_name, context is $context_class";
            }
            $context->$method_name($params);
        }
        else {
            die "cannot call $method_name on $class_name, no context";
        }
    }

    return { instance_id => $context->id };
}

with qw/
    state51::Mixin::JobLogger
/;

__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
