package state51::Job::RaiseFlagsForQAAdvisor;
use state51::Base;
use MooseX::Types::Moose qw/ ArrayRef Str Int HashRef Maybe/;
use MooseX::Types::Varchar qw/ Varchar /;
use Try::Tiny;
use Module::Find qw/ useall /;
use Carp;
use Moose;
use Moose::Util::TypeConstraints;
extends 'state51::Job';

my @qa_advisor_rules;

BEGIN {
    @qa_advisor_rules = useall state51::QAAdvisorRule;
}

has 'advisor_class' => (
    isa => Str,
    is  => 'ro',
    documentation =>
'QAAdvisorRule class to raise flags for, e.g. "state51::QAAdvisorRule::ITMSStyle"'
);

has 'format_oid' => (
    isa     => Int,
    is      => 'ro',
    default => sub { [] },
    documentation =>
'ReleaseFormat OID to raise QA advisor flags against. This is a full instance id including the class id on the end.'
);

has 'flags_to_raise' => (
    isa => ArrayRef [HashRef],
    is => 'ro',
    default => sub { [] },
    documentation =>
'Array of hashefs that can be used to instantiate QANotes. Each ref must contain "flag",'
      . 'which is the flag id to raise. It can optionally contain track and volume,'
      . 'to relate the flag to a specific track. Note that this is the complete set of flags '
      . 'for this advisory class that should be open. Any other flags currently open for this class '
      . 'will be closed.'
);

sub exchange_name {
    'raiseflagsforqaadvisor';
}

has watchlist_match_detail => (
    is       => 'rw',
    isa      => Varchar [600],
    required => 0,
    default  => undef,
    documentation =>
'Holds a user facing message detailing why a watchlist QA flag was raised',
);

has qa_note => (
    is       => 'rw',
    isa      => Maybe[HashRef],
    required => 0,
    default  => undef,
    documentation =>
'Holds the track ID and volume',
);

class_type 'state51::AudibleMagicMatch';
has audio_matches => (
    is       => 'rw',
    isa      => Maybe[ArrayRef [HashRef]],
    required => 0,
    default  => undef,
    documentation =>
'Array of hashrefs that carry details of audio matches.',
);

sub suppress_console_display { 1 }

sub run {
    my ($self) = @_;
    state51::Base->connect;

    $self->raise_flags_for_advisor($self->advisor_class, $self->format_oid,
        $self->flags_to_raise);

    return 1;
}

sub raise_flags_for_advisor {
    my ($self, $advisor_class, $format_id, $flat_flags_to_raise) = @_;

    my $format  = state51::ReleaseFormat->load($format_id);
    my $advisor = $advisor_class->new();

    my @flag_type_to_raise;
    # my @audio_matches_obj;
    foreach my $flag_to_raise (@{$flat_flags_to_raise}) {
        my %args;
        $args{flag}    = $flag_to_raise->{flag};
        $args{advisor} = $advisor;

        if ($flag_to_raise->{track}) {
            $args{track}  = $flag_to_raise->{track};
            $args{volume} = $flag_to_raise->{volume};
        }

        push @flag_type_to_raise, state51::QANote->new(%args);

        # if ($audio_matches) { # for AUDIBLEMAGICMATCH flags
        #     foreach my $audio_match (@{$audio_matches}) {
        #         my %match_args;
        #         $match_args{isrc}                      = $audio_match->{ISRC};
        #         $match_args{label}                     = $audio_match->{Label};
        #         $match_args{vendor}                    = $audio_match->{Vendor};
        #         $match_args{album_release_date}        = $audio_match->{AlbumReleaseDate};
        #         $match_args{album_upc}                 = $audio_match->{AlbumUPC};
        #         $match_args{title}                     = $audio_match->{Title};
        #         $match_args{artist}                    = $audio_match->{Artist};
        #         $match_args{match_time_in_unknown}     = $audio_match->{matchTimeInUnknown};
        #         $match_args{match_duration_in_unknown} = $audio_match->{matchDurationInUnknown};
        #         $match_args{match_time_in_reference}   = $audio_match->{matchTimeInReference};
        #         # my $audio_match_obj = state51::AudibleMagicMatch->new(
        #         #     isrc                      => $audio_match->{ISRC};
        #         #     label                     => $audio_match->{Label};
        #         #     vendor                    => $audio_match->{Vendor};
        #         #     album_release_date        => $audio_match->{AlbumReleaseDate};
        #         #     album_upc                 => $audio_match->{AlbumUPC};
        #         #     title                     => $audio_match->{Title};
        #         #     artist                    => $audio_match->{Artist};
        #         #     match_time_in_unknown     => $audio_match->{matchTimeInUnknown};
        #         #     match_duration_in_unknown => $audio_match->{matchDurationInUnknown};
        #         #     match_time_in_reference   => $audio_match->{matchTimeInReference};
        #         # );

        #         # push @audio_matches_obj, $audio_match_obj;
        #         push(@audio_matches_obj, state51::AudibleMagicMatch->new(%match_args));
        #     }
        # }
    }

    my @current_open_flags = state51::QAFlag->fetch(release_format => $format);
    my %remaining_flags;

    if (@flag_type_to_raise) {
        foreach my $flag_obj (@flag_type_to_raise) {
            my $found_flag = $self->ensure_is_flagged($flag_obj, $format);

            $remaining_flags{$found_flag->id} = 1;
        }
    }

    foreach my $flag (@current_open_flags) {
        if (  !$remaining_flags{$flag->id}
            && $advisor->can_i_raise($flag->qa_flag_type->qa_advice_id))
        {
            $flag->erase();
        }
    }

    $format->qa_flags_run(state51::Base->dbnow);
    $format->update;
}

sub ensure_is_flagged {
    my ($self, $flag_obj, $format) = @_;

    my $flag_type = $flag_obj->flag;
    my $type = state51::QAFlagType->fetch_one(qa_advice_id => $flag_type);
    my $found_flag;

    if ($flag_obj->is_track) {

        # doesn't matter if it's closed
        ($found_flag) = state51::QAFlag::Track->fetch(
            release_format => $format,
            qa_flag_type   => $type,
            volume_part    => $flag_obj->volume,
            track_number   => $flag_obj->track,
            # audible_magic_matches => @audio_matches_obj,
        );
        if (not $found_flag) {
            $found_flag = state51::QAFlag::Track->new(
                release_format => $format,
                qa_flag_type   => $type,
                volume_part    => $flag_obj->volume,
                track_number   => $flag_obj->track,
                # audible_magic_matches => @audio_matches_obj,
            );
            $found_flag->insert();
        }
    }
    else {
        # doesn't matter if it's closed
        ($found_flag) = state51::QAFlag->fetch(
            release_format => $format,
            qa_flag_type   => $type,
        );
        if (not $found_flag) {
            $found_flag = state51::QAFlag->new(
                release_format => $format,
                qa_flag_type   => $type
            );
            $found_flag->insert();
        }

        if ($self->watchlist_match_detail) {
            $found_flag->watchlist_match_detail($self->watchlist_match_detail);
            $found_flag->update();
        }

        if ($self->qa_note) {
            $found_flag->qa_note($self->qa_note);
            $found_flag->update();
        }

        if ($self->audio_matches) {
            my @audio_matches_obj;
            foreach my $audio_match ($self->audio_matches) {
                my %match_args;
                $match_args{isrc}                      = $audio_match->{ISRC};
                $match_args{label}                     = $audio_match->{Label};
                $match_args{vendor}                    = $audio_match->{Vendor};
                $match_args{album_release_date}        = $audio_match->{AlbumReleaseDate};
                $match_args{album_upc}                 = $audio_match->{AlbumUPC};
                $match_args{title}                     = $audio_match->{Title};
                $match_args{artist}                    = $audio_match->{Artist};
                $match_args{match_time_in_unknown}     = $audio_match->{matchTimeInUnknown};
                $match_args{match_duration_in_unknown} = $audio_match->{matchDurationInUnknown};
                $match_args{match_time_in_reference}   = $audio_match->{matchTimeInReference};
                # my $audio_match_obj = state51::AudibleMagicMatch->new(
                #     isrc                      => $audio_match->{ISRC};
                #     label                     => $audio_match->{Label};
                #     vendor                    => $audio_match->{Vendor};
                #     album_release_date        => $audio_match->{AlbumReleaseDate};
                #     album_upc                 => $audio_match->{AlbumUPC};
                #     title                     => $audio_match->{Title};
                #     artist                    => $audio_match->{Artist};
                #     match_time_in_unknown     => $audio_match->{matchTimeInUnknown};
                #     match_duration_in_unknown => $audio_match->{matchDurationInUnknown};
                #     match_time_in_reference   => $audio_match->{matchTimeInReference};
                # );

                # push @audio_matches_obj, $audio_match_obj;
                push(@audio_matches_obj, state51::AudibleMagicMatch->new(%match_args));
            }
            $found_flag->audio_matches(@audio_matches_obj);
            $found_flag->update();
        }
    }
    return $found_flag;
}

__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
