package state51::Job::RetryTransferForBatch;
use state51::Base;
use state51::Moose;
use MooseX::Storage;
with Storage (format => "JSON");

use Try::Tiny;
use Moose::Util::TypeConstraints;
use MooseX::Types::Common::String qw/ NonEmptySimpleStr /;
use Moose::Autobox;
use Capture::Tiny qw/ capture_merged /;
use MooseX::Types::DateTime::MySQL ();
use MooseX::Types::DateTime qw/ DateTime /;
use DateTime::Duration;
use aliased DateTime => 'DT';
use namespace::autoclean;
use Data::UUID;

extends 'state51::Job';

has 'batch_id' => (
    is            => 'ro',
    isa           => NonEmptySimpleStr,
    required      => 1,
    documentation => 'A batch id',
);

class_type 'state51::ExportBatch';
has _batch => (
    is => 'ro',
    isa => 'state51::ExportBatch',
    traits => [qw/ DoNotSerialize /],
    default => sub {
        my $self = shift;
        state51::Base->connect;
        state51::ExportBatch->load($self->batch_id);
    },
    clearer => '_clear_batch',
    lazy => 1,
);

method exchange_name() {
    my $suffix = "";
    $suffix = "_premium" if ($self->_batch->to_fulfil->priority eq "Premium");
    return 'ordertransfer_retry_' . lc($self->_batch->exporter_hostname) . $suffix;
}

method lockname() {
    $self->_batch->musicservice->serialize_deliveries
        ? "order_transfer_".$self->_batch->musicservice->id
        : "order_transfer_".$self->_batch->id;
}

method command_line() {
    return " for " . $self->_batch->to_fulfil->id . " to " . $self->_batch->musicservice->tla . " (" . $self->batch_id . ")";
}

my $ug = Data::UUID->new;
has uuid => (
    is => 'ro',
    default => sub { my $uuid = $ug->create; $ug->to_string($uuid) },
);

method run() {
    state51::Base->connect;
    my $class = ref($self);
    $class =~ s/state51::Job:://;
    confess("HUH, should be impossible") unless $self->_batch->is_persistent;
    my $lockname = $self->lockname;
    $self->do_whilst_locked($lockname,
        sub {
            if ( $self->_batch->status eq 'complete'
                        and ($self->_batch->service_status eq 'transferred'
                        or $self->_batch->service_status eq 'transferring')
            ) {
                $self->log_message(level => "warn", message => ["Batch id:" . $self->batch_id . " is complete and is transferred or transferring"]) if $self->interactive;
                return 1;
            }
            if ( $self->_batch->status ne 'complete' ) {
                $self->log_message(level => "warn", message => ["Batch id:" . $self->batch_id . " is not complete"]) if $self->interactive;
                return 1;
            }
            if ( $self->_batch->to_fulfil->status ne 'ready' ) {
                $self->log_message(level => "warn", message => ["Batch id:" . $self->batch_id . ". Order is not ready"]) if $self->interactive;
                return 1;
            }
            my $comments = '';
            my ($has_error, $has_exception);
            my $my_hostname = (split('\.',Sys::Hostname::hostname()))[0];
            my ($out, $fh, $order);
            if (not $self->_batch->musicservice) {
                $comments .= "Batch '".$self->_batch->id."' doesn't have a musicservice, cannot continue. ";
                $has_error = 1;
            }
            elsif (not $self->_batch->to_fulfil) {
                $comments .= "Export batch is doesn't relate to an export request. Can't redeliver. ";
                $has_error = 1;
            }
            else {
                $order = $self->_batch->to_fulfil;
                $comments = sprintf("\nRetry of batch %s started at %s\n", $self->_batch->id, $self->date);

                my $log_dir = $self->_batch->musicservice->export_path.'/log';
                mkdir $log_dir unless -d $log_dir;
                my $log_path = $log_dir . '/'.$order->id;

                open $fh, '>>', $log_path
                    or $self->log_message(level => "warn", message => ["Batch id:" . $self->batch_id . ". Cannot write to $log_path: $!"]);
                chmod 0664, $log_path;

                my $retry_limit = 3;
                $out = capture_merged {
                    try {
                        $self->_batch->delivery_job_uuid($self->uuid);
                        $self->_batch->update;
                        if ( $self->_batch->upload_attempts <= $retry_limit ) {
                            try {
                                my $musicservice = $self->_batch->musicservice->musicservice_class_instance;

                                $musicservice->export_request($order);
                                $musicservice->export_batch($self->_batch);
                                $musicservice->set_handler_defaults();
                                if ($musicservice->can('set_export_working_paths')) {
                                    $musicservice->set_export_working_paths();
                                }
                                $musicservice->transport_and_notify();
                                $self->_clear_batch; # Force reload, delivery could have taken ages..
                                unless ($self->_batch->status eq "complete" && $self->_batch->service_status eq "transferred") {
                                    $has_error = 1;
                                    $comments .= "\n\nERROR: Export batch " . $self->_batch->id . " is in status " . $self->_batch->status . " transfer status " . $self->_batch->service_status . "\n";
                                }
                                else {
                                    $comments .= "Batch transfer finished successfully";
                                    $self->_batch->update;
                                }

                            }
                            catch {
                                my $error = $_;
                                my $error_class = blessed($error);
                                $self->_clear_batch; # Force reload, delivery could have taken ages..
                                $comments .= "'$my_hostname' found an error during RetryTransferForBatch".$self->_batch->musicservice->tla . "\n$error\n";
                                if ($error_class eq 'ExFatalTransportError') {
                                    $self->_batch->service_status('error');
                                    $self->_batch->update;
                                }
                                $has_error = 1;
                            };
                        }
                        else {
                            $comments .= sprintf("Automatic content deliver retries limit reached. We tried delivery %s times of %s allowed",
                            $self->_batch->upload_attempts, $retry_limit);
                            $has_error = 1;
                            $self->_batch->service_status('error');
                            $self->_batch->update;
                        }
                    }
                    catch {
                        $comments .= "CAUGHT EXCEPTION: $_";
                        if (/Has it been deleted/) { # Eww, gross hack..
                            $comments .= "\nAUTO JUNKING BATCH WHICH HAS BEEN DELETED SO IT IS RE-COMPOSED\n";
                            $self->_batch->status('junk');
                            $self->_batch->update;
                            $self->_batch->remove_from_disk;
                        }
                        else {
                            $has_exception = 1;
                        }
                        $has_error = 1;
                    };
                };

                # Dump output from script also.
                $comments .= $out;

                try {
                    $order->update_ticket_status('open') if $has_error;
                }
                catch {
                    $self->log_message(level => "warn", message => ["Batch id:" . $self->batch_id . ". RetryTransferForBatch: Error adding comment in RT: $_\n".$comments]);
                };

                print "\n---------------------------\n";
                print $fh "\n---------------------------\n" if $fh;
                print $comments;
                print $fh $comments if $fh;

                $self->run_job('state51::Job::PostTransferStatusUpdate', batch_id => $self->batch_id);
                return ! $has_exception;
            }
        }
    )
    || do { if ($self->interactive) { $self->log_message(level => "warn", message => ["Batch id:" . $self->batch_id . ". Already locked: $lockname"]) } };
    return 1;
}

with qw/
    state51::Script::Helper::InteractiveSuccess
    state51::Mixin::Loggable
    state51::Mixin::JobSignature
    state51::Mixin::RunsAsyncJobs
    state51::Mixin::HasLocks
    state51::Mixin::JobLogger
/;

#
__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
