package state51::Job::RunQAForFormat;
use state51::Moose;
use MooseX::Storage;
with Storage (format => "JSON");

use state51::Base;
use state51::RT;
use MooseX::Types::Moose qw/ ArrayRef Str Int /;
use Try::Tiny;
use Module::Find qw/ useall /;
use Carp;
use MooseX::Storage;

extends 'state51::Job';

with Storage(format => 'JSON');

my @qa_advisor_rules;
BEGIN {
    @qa_advisor_rules = useall state51::QAAdvisorRule;
}

has 'format_oid' => (
    isa => ArrayRef[Int],
    is => 'ro',
    default => sub { [] },
    documentation => 'OIDs to run QA advisors against'
);

sub exchange_name {
    'runqaforformat'
}

sub suppress_console_display { 1 }

sub run {
    my ($self) = @_;
    state51::Base->connect;

    foreach my $oid (@{ $self->format_oid }) {
        my $format = state51::ReleaseFormat->load($oid);
        $format->suppress_qa_run(1);
        $self->run_qa_for_format($format);
        $format->suppress_qa_run(0);
    }
}

method run_qa_for_format ($format) {
    if ($format->asset_controller->suppress_qa_flags) {
        $format->qa_flags_run(state51::Base->dbnow);
        $format->update;
        return;
    }

    foreach my $advisor_class (@qa_advisor_rules) {
        my $advisor = $advisor_class->new();

        next if $advisor->implemented_elsewhere();

        my $error;
        my @flag_type_to_raise;
        try {
            @flag_type_to_raise = $advisor->raise_advice_for($format);
        }
        catch {
            $error = $_;
            warn $error;
            # FIXME: not an RT ticket, replace with a log event
            # $self->raise_rt_ticket($error, $format);
        };

        # need to flatten flags_to_raise into something that can go
        # over the wire, it's currently QANote
        my @flat_flags_to_raise;
        foreach my $flag_to_raise (@flag_type_to_raise) {
            push @flat_flags_to_raise, {
                    flag  => $flag_to_raise->flag,
                    track => $flag_to_raise->track,
                    volume => $flag_to_raise->volume,
            }
        }
        $self->raise_flags_for_advisor($advisor_class, $format->id, \@flat_flags_to_raise);
    }
}

method raise_flags_for_advisor($advisor_class, $format_id, $flat_flags_to_raise) {
    my $format = state51::ReleaseFormat->load($format_id);
    my $advisor = $advisor_class->new();

    my @flag_type_to_raise;
    foreach my $flag_to_raise (@{$flat_flags_to_raise}) {
        my %args;
        $args{flag} = $flag_to_raise->{flag};
        $args{advisor} = $advisor;

        if ($flag_to_raise->{track}) {
            $args{track} = $flag_to_raise->{track};
            $args{volume} = $flag_to_raise->{volume};
        }

        push @flag_type_to_raise, state51::QANote->new(%args);
    }

	my @current_open_flags = state51::QAFlag->fetch(release_format => $format);
	my %remaining_flags;

	if (@flag_type_to_raise) {
		foreach my $flag_obj (@flag_type_to_raise) {
			my $found_flag = $self->ensure_is_flagged($flag_obj, $format);

			$remaining_flags{$found_flag->id} = 1;
		}
	}

	foreach my $flag (@current_open_flags) {
		if ( !$remaining_flags{$flag->id}
			&&$advisor->can_i_raise($flag->qa_flag_type->qa_advice_id)) {
			$flag->erase();
		}
	}

    $format->qa_flags_run(state51::Base->dbnow);
    $format->update;
}

method ensure_is_flagged ($flag_obj, $format) {
    my $flag_type = $flag_obj->flag;
    my $type = state51::QAFlagType->fetch_one(qa_advice_id => $flag_type);
    my $found_flag;

    if ($flag_obj->is_track) {
        # doesn't matter if it's closed
        ($found_flag) = state51::QAFlag::Track->fetch(release_format => $format,
                                                      qa_flag_type   => $type,
                                                      volume_part    => $flag_obj->volume,
                                                      track_number   => $flag_obj->track,
                                                  );
        if (not $found_flag) {
            $found_flag = state51::QAFlag::Track->new(release_format => $format,
                                                      qa_flag_type   => $type,
                                                      volume_part    => $flag_obj->volume,
                                                      track_number   => $flag_obj->track,
                                                  );
            $found_flag->insert();
        }
    }
    else {
        # doesn't matter if it's closed
        ($found_flag) = state51::QAFlag->fetch(release_format => $format,
                                               qa_flag_type   => $type,
                                           );
        if (not $found_flag) {
            $found_flag = state51::QAFlag->new(release_format => $format,
                                               qa_flag_type   => $type
                                           );
            $found_flag->insert();
        }
    }
    return $found_flag;
}

with qw/
    
    state51::Mixin::Loggable
    state51::Mixin::JobLogger
    state51::Mixin::HasLogger
    state51::Mixin::UnloadAfterJob
/;

__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
