package state51::Job::ValidateDeliverDesires;
use state51::Base;
use state51::Moose;
use MooseX::Storage;
with Storage( format => "JSON" );

use Try::Tiny;
use MooseX::Types::Moose qw/ Int Bool Undef/;
use MooseX::Types::Common::String qw/ NonEmptySimpleStr /;
use namespace::autoclean;
use Lingua::EN::Inflect qw/PL/;

extends 'state51::Job';

sub exchange_name { 'orderxlsvalidate' }

method run() {
    my $correspondence;
    my $exception      = 0;
    my $export_request = $self->_order_object;
    my $metadata_validated =
      !$export_request->require_service_metadata_xls_validation;
    my $ppl_data_validated =
      !$export_request->require_ppl_metadata_xls_validation;
    my $comments =
        'Validated Deliver Desires for order '
      . $self->order
      . ( ( $metadata_validated == 0 || $ppl_data_validated == 0 )
        ? ' and attached file'
        : ' with no attached file' );
    $comments .= "\n";
    my $no_deliverable_upcs = {};
    my $invalid_dps         = {};
    my $errors_txt          = '';
    my $metadata_errors;
    my $ppl_errors;

    return unless ( $export_request->status eq 'validating' );

    #
    # Per-Service Offer application
    #
    # Must be done before process_service_metadata_xls, as this will try
    # applying per-service pre-order dates (on conditional offer rels).
    try {
        my $order_raising_org = $export_request->organisation();
        my $pso = state51::ImportFormat::PerServiceOffers->new;
        my $restrictions = $pso->get_restrictions(
            $order_raising_org->identifier
        );

        if ($restrictions) {
            my @formats = $export_request->get_formats_to_deliver;
            foreach my $fmt (@formats) {
                if (not $fmt->highest_common_encoding) {
                    eval {
                        $fmt->reset_highest_common_audio_encoding();
                    };
                }
                $pso->apply_per_service_offers_if_needed_to_asset(
                    asset         => $fmt,
                    musicservices => $export_request->musicservice_destinations,
                );

                $_->unload for @{ $fmt->ordered_tracks };
                $fmt->unload;
            }
        }
    }
    catch {
        $exception = 1;
        $errors_txt .=
          "\nError preparing availability; we are looking into this.";
        $comments .= "\nException applying Per-Service Offers: \n" . $_;
    };

    #
    # Service Metadata (per-product price-tiers and pre-order dates)
    #

    try {
        if ( not $metadata_validated ) {
            $metadata_validated = 0;
            $metadata_errors    = $export_request->process_service_metadata_xls;
            if ( ref $metadata_errors eq 'ARRAY' && scalar(@$metadata_errors) )
            {
                $exception = 1;    #errors processing metadata xls
                $comments .=
                  "\nException processing service Metadata XLS (order id "
                  . $self->order . "):\n";
                foreach (@$metadata_errors) {
                    $comments .= "\t- $_\n";
                }
            }
            elsif ( ref $metadata_errors eq 'HASH'
                && scalar( keys %$metadata_errors ) )
            {
                $metadata_validated = 0;    # errors validating metadata xls
            }
            else {
                $metadata_validated = 1;
            }
        }
    }
    catch {
        $exception = 1;
        $errors_txt .=
"\nSystem failure checking Metadata Service XLS, we are looking into this.";
        $comments .= "\nException checking Metadata Service XLS: \n" . $_;
    };

    #
    # PPL Metadata
    #

    try {
        if ( not $ppl_data_validated ) {
            $ppl_errors = $export_request->process_ppl_metadata_xls;
            if ( ref $ppl_errors eq 'HASH' && scalar( keys %$ppl_errors ) ) {
                $ppl_data_validated = 0;
            }
            else {
                $ppl_data_validated = 1;
            }
        }
    }
    catch {
        $errors_txt .=
"\nSystem failure checking PPL Service XLS, we are looking into this.";
        $comments .=
            "Exception validating PPL Metadata XLS (order id "
          . $self->order
          . " file_no "
          . $ppl_data_validated
          . "): $_\n";
        $exception = 1;
    };

    #
    # Validating Delivery Desires
    #

    try {
        $no_deliverable_upcs = $export_request->validate_order_desires;
        $errors_txt .= $export_request->describe_desire_validation_errors($no_deliverable_upcs) if scalar keys %$no_deliverable_upcs;
        $errors_txt .= "\n\n".$export_request->describe_delivery_point_validation_errors($invalid_dps) if scalar keys %$invalid_dps;
    }
    catch {
        $exception = 1;
        $errors_txt .=
          "\nSystem failure checking order, we are looking into this.";
        $comments .= "\nException checking deliver desires: \n" . $_;
    };

    #
    # Cleaning up - various notifications about the outcomes of the above
    #

    my $ticket_status = 'stalled';
    my $status        = 'ready';

    if ( not $exception ) {
        if ( scalar keys %$no_deliverable_upcs ) {
            if ( $export_request->number_of_products ) {
                $correspondence = sprintf(
"Some products have been automatically removed from this delivery request as they are not able to be sent:\n%s\n\nSee below for the details.\n\n%s\n%s\n\n",
                    join( ', ', keys %$no_deliverable_upcs ),
                    ($metadata_validated)
                    ? "The rest of this delivery order will now proceed.\n(No further action is needed on your part.)"
                    : "",
                    $errors_txt
                );
            }
            else {
                $status         = 'pending';
                $correspondence = sprintf(
"Products below cannot be delivered:\n%s\n\nThe order cannot proceed, please fix the errors found and post the order again.\n%s\n\n",
                    join( ', ', keys %$no_deliverable_upcs ), $errors_txt );
                $export_request->reset_invalid_dd()
                  ; #reset invalid DD to pending so in next order post they are validated again
            }
        }
        elsif ($metadata_validated) {
            $correspondence = sprintf(
"\nThis order has validated correctly and delivery will proceed for %s products (%s tracks).\n",
                $export_request->number_of_products,
                $export_request->number_of_tracks
            );
        }

        if ( not $metadata_validated || not $ppl_data_validated ) {
            $status = 'pending';
            if ( ref $metadata_errors eq 'HASH'
                && scalar( keys %$metadata_errors ) )
            {
                $correspondence .=
"There are errors on the Service Metadata XLS attached, please check the errors below, upload a new Service Metadata XLS and post the order again:"
                  . $export_request->describe_validation_errors(
                    $metadata_errors);
            }
            if ( ref $ppl_errors eq 'HASH' && scalar( keys %$ppl_errors ) ) {
                $correspondence .=
"\n\nThere are errors on the Service metadata spreadsheet attached, please check the errors below, upload a new PPL data XLS and post the order again:";
                foreach my $row_number ( keys %$ppl_errors ) {
                    $correspondence .=
                      "\n  - Row: $row_number contains invalid data: "
                      . $ppl_errors->{$row_number};
                }
            }
        }

        if ( $status eq 'ready' ) {
            $self->log_message(
                level   => 'info',
                message => [
                    state51::Job::RequestTransformsForOrder->new(
                        order => $self->order,
                    )
                ],
                nodb => 1,
            );
            $comments .= "Transforms have been requested.";
        }
        my $count_undeliverable_upcs = scalar( keys(%$no_deliverable_upcs) );
        $comments .=
            "\n --------------- \n - "
          . PL( "Product", $export_request->number_of_products )
          . " to deliver: "
          . $export_request->number_of_products . "\n"
          . "- undeliverable "
          . PL( 'UPC', $count_undeliverable_upcs ) . ': '
          . $count_undeliverable_upcs
          . "\n - metadata validated: $metadata_validated\n";

        $export_request->_set_status($status);
        $export_request->update;
    }
    else {
        $status        = 'validating';
        $ticket_status = 'open';
    }
    try {
        $self->correspond_in_rt($correspondence) if ($correspondence);
        $self->update_ticket_status($ticket_status);
    }
    catch {
        $comments .= "ValidateDeliverDesires: ERROR COMMENTING IN RT $_";
        warn($comments);
    };
    print $comments if $self->interactive;

    return !$exception;
}

method lockname() { "order_validate_" . $self->order }

with qw/
  state51::Mixin::Loggable
  state51::Mixin::HandleOrdersOnJobs
  state51::Mixin::RunsAsyncJobs
  state51::Mixin::JobSignature
  state51::Script::Helper::InteractiveSuccess
  state51::Mixin::LockedJob
  state51::Mixin::JobLogger
  /;

#
# __PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
