package state51::Job::WithdrawFormatsForOldArchivedOrgs;
use Moose;

use MooseX::Types::Moose qw/ Str Bool /;
use state51::Types qw/InstanceID/;
use Moose::Util::TypeConstraints;
use MooseX::ClassAttribute;
use MooseX::Types::DateTime qw/ DateTime /;
use MooseX::Types::DateTime::MySQL qw/ MySQLDateTime /;
use List::Util qw(shuffle);

extends 'state51::Job';

class_has documentation => (
    is  => 'ro',
    isa => 'Str',
    default =>
'Find archived media providers not modified in a month with a live format and withdraw all formats.',
);

# needed for RunsAsyncJobs
has interactive => (
    is => 'ro',
    isa => 'Bool',
    default => 0,
);

sub exchange_name {
    'withdraw_formats_for_old_archived_orgs';
}

sub suppress_console_display { 1 }

sub run {
    my ($self) = @_;
    state51::Base->connect;

    # finds any org which is archived, a media provider, not modified in the past month, which has a live release format.
    # (if you can do this in a single query you win a prize)
    # then withdraws all formats from that org. by raising job WithdrawAllFormatsForAC.

    my $one_month_ago =  to_MySQLDateTime( DateTime->now->subtract(months => 1) );

    my $media_provider = state51::OrgType->fetch_one(name => 'Media Provider');

    my $org_remote = state51::Organisation->remote();
    my $format_remote = state51::ReleaseFormat->remote();

    my $filter = ($org_remote->{type} eq $media_provider &
          $org_remote->{status} eq 'archived' &
          $org_remote->{modify_time} lt $one_month_ago
    );
    my @old_withdrawn_orgs = state51::Base->select_readonly(
        $org_remote,
        filter => $filter,
        distinct => 1,
    );

    my %seen_orgs;
    foreach my $org (shuffle @old_withdrawn_orgs) {
        next if $seen_orgs{$org};

        my ($any_live_format) = state51::ReleaseFormat->fetch(asset_controller => $org, status => 'live', limit => 1);
        next unless $any_live_format;

        $self->logger->message(
            level   => "info",
            message => [
                state51::Logger::Event::Generic->new(
                    message => "Withdrawing all formats for ".$org->name." ".$org->id()
                )
            ]
        );
        $self->run_job(
            'state51::Job::WithdrawAllFormatsForAC',
            asset_controller_id => $org->id(),
            async             => 1,
        );
    }

    return 1;
}

with qw/
  state51::Mixin::JobLogger
  state51::Mixin::HasLogger
  state51::Mixin::RunsAsyncJobs
  /;

__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
